@@ -38,7 +38,6 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.hbase.Cell.Type;
 import org.apache.hadoop.hbase.client.Connection;
 import org.apache.hadoop.hbase.client.Consistency;
 import org.apache.hadoop.hbase.client.Delete;
@@ -155,6 +154,7 @@ public class MetaTableAccessor {
   private static final byte SEPARATED_BYTE = 0x00;
 
   @InterfaceAudience.Private
+  @SuppressWarnings("ImmutableEnumChecker")
   public enum QueryType {
     ALL(HConstants.TABLE_FAMILY, HConstants.CATALOG_FAMILY),
     REGION(HConstants.CATALOG_FAMILY),
@@ -289,7 +289,8 @@ public class MetaTableAccessor {
     RegionLocations locations = getRegionLocations(r);
     return locations == null
       ? null
-      : locations.getRegionLocation(parsedInfo == null ? 0 : parsedInfo.getReplicaId());
+      : locations.getRegionLocation(
+        parsedInfo == null ? RegionInfo.DEFAULT_REPLICA_ID : parsedInfo.getReplicaId());
   }
 
   /**
@@ -333,12 +334,12 @@ public class MetaTableAccessor {
   /**
    * Gets the result in hbase:meta for the specified region.
    * @param connection connection we're using
-   * @param regionName region we're looking for
+   * @param regionInfo region we're looking for
    * @return result of the specified region
    */
-  public static Result getRegionResult(Connection connection, byte[] regionName)
+  public static Result getRegionResult(Connection connection, RegionInfo regionInfo)
     throws IOException {
-    Get get = new Get(regionName);
+    Get get = new Get(getMetaKeyForRegion(regionInfo));
     get.addFamily(HConstants.CATALOG_FAMILY);
     return get(getMetaHTable(connection), get);
   }
@@ -364,25 +365,25 @@ public class MetaTableAccessor {
   }
 
   /**
-   * @return Return all regioninfos listed in the 'info:merge*' columns of the
-   *         <code>regionName</code> row.
+   * Returns Return all regioninfos listed in the 'info:merge*' columns of the {@code regionInfo}
+   * row.
    */
   @Nullable
-  public static List<RegionInfo> getMergeRegions(Connection connection, byte[] regionName)
+  public static List<RegionInfo> getMergeRegions(Connection connection, RegionInfo regionInfo)
     throws IOException {
-    return getMergeRegions(getRegionResult(connection, regionName).rawCells());
+    return getMergeRegions(getRegionResult(connection, regionInfo).rawCells());
   }
 
   /**
-   * Check whether the given {@code regionName} has any 'info:merge*' columns.
+   * Check whether the given {@code regionInfo} has any 'info:merge*' columns.
    */
-  public static boolean hasMergeRegions(Connection conn, byte[] regionName) throws IOException {
-    return hasMergeRegions(getRegionResult(conn, regionName).rawCells());
+  public static boolean hasMergeRegions(Connection conn, RegionInfo regionInfo) throws IOException {
+    return hasMergeRegions(getRegionResult(conn, regionInfo).rawCells());
   }
 
   /**
-   * @return Deserialized values of &lt;qualifier,regioninfo&gt; pairs taken from column values that
-   *         match the regex 'info:merge.*' in array of <code>cells</code>.
+   * Returns Deserialized values of &lt;qualifier,regioninfo&gt; pairs taken from column values that
+   * match the regex 'info:merge.*' in array of <code>cells</code>.
    */
   @Nullable
   public static Map<String, RegionInfo> getMergeRegionsWithName(Cell[] cells) {
@@ -408,8 +409,8 @@ public class MetaTableAccessor {
   }
 
   /**
-   * @return Deserialized regioninfo values taken from column values that match the regex
-   *         'info:merge.*' in array of <code>cells</code>.
+   * Returns Deserialized regioninfo values taken from column values that match the regex
+   * 'info:merge.*' in array of <code>cells</code>.
    */
   @Nullable
   public static List<RegionInfo> getMergeRegions(Cell[] cells) {
@@ -418,8 +419,8 @@ public class MetaTableAccessor {
   }
 
   /**
-   * @return True if any merge regions present in <code>cells</code>; i.e. the column in
-   *         <code>cell</code> matches the regex 'info:merge.*'.
+   * Returns True if any merge regions present in <code>cells</code>; i.e. the column in
+   * <code>cell</code> matches the regex 'info:merge.*'.
    */
   public static boolean hasMergeRegions(Cell[] cells) {
     for (Cell cell : cells) {
@@ -483,6 +484,7 @@ public class MetaTableAccessor {
     return getListOfRegionInfos(result);
   }
 
+  @SuppressWarnings("MixedMutabilityReturnType")
   private static List<RegionInfo>
     getListOfRegionInfos(final List<Pair<RegionInfo, ServerName>> pairs) {
     if (pairs == null || pairs.isEmpty()) {
@@ -496,8 +498,7 @@ public class MetaTableAccessor {
   }
 
   /**
-   * @param tableName table we're working with
-   * @return start row for scanning META according to query type
+   * Returns start row for scanning META according to query type
    */
   public static byte[] getTableStartRowForMeta(TableName tableName, QueryType type) {
     if (tableName == null) {
@@ -518,8 +519,7 @@ public class MetaTableAccessor {
   }
 
   /**
-   * @param tableName table we're working with
-   * @return stop row for scanning META according to query type
+   * Returns stop row for scanning META according to query type
    */
   public static byte[] getTableStopRowForMeta(TableName tableName, QueryType type) {
     if (tableName == null) {
@@ -641,6 +641,7 @@ public class MetaTableAccessor {
   }
 
   /**
+   * Get the user regions a given server is hosting.
    * @param connection connection we're using
    * @param serverName server whose regions we're interested in
    * @return List of user regions installed on this server (does not include catalog regions).
@@ -1284,7 +1285,7 @@ public class MetaTableAccessor {
       if (info == null) {
         return true;
       }
-      if (!(info.getTable().equals(tableName))) {
+      if (!info.getTable().equals(tableName)) {
         return false;
       }
       return super.visit(rowResult);
@@ -1294,11 +1295,30 @@ public class MetaTableAccessor {
   ////////////////////////
   // Editing operations //
   ////////////////////////
+
   /**
-   * Generates and returns a Put containing the region into for the catalog table
+   * Generates and returns a {@link Put} containing the {@link RegionInfo} for the catalog table.
+   * @throws IllegalArgumentException when the provided RegionInfo is not the default replica.
+   */
+  public static Put makePutFromRegionInfo(RegionInfo regionInfo) throws IOException {
+    return makePutFromRegionInfo(regionInfo, EnvironmentEdgeManager.currentTime());
+  }
+
+  /**
+   * Generates and returns a {@link Put} containing the {@link RegionInfo} for the catalog table.
+   * @throws IllegalArgumentException when the provided RegionInfo is not the default replica.
    */
   public static Put makePutFromRegionInfo(RegionInfo regionInfo, long ts) throws IOException {
-    return addRegionInfo(new Put(regionInfo.getRegionName(), ts), regionInfo);
+    byte[] metaKeyForRegion = getMetaKeyForRegion(regionInfo);
+    try {
+      Put put = new Put(metaKeyForRegion, ts);
+      return addRegionInfo(put, regionInfo);
+    } catch (IllegalArgumentException ex) {
+      LOG.error(
+        "Got exception while creating put for regioninfo {}." + "meta key for regioninfo is {}",
+        regionInfo.getRegionNameAsString(), metaKeyForRegion);
+      throw ex;
+    }
   }
 
   /**
@@ -1308,7 +1328,11 @@ public class MetaTableAccessor {
     if (regionInfo == null) {
       throw new IllegalArgumentException("Can't make a delete for null region");
     }
-    Delete delete = new Delete(regionInfo.getRegionName());
+    if (regionInfo.getReplicaId() != RegionInfo.DEFAULT_REPLICA_ID) {
+      throw new IllegalArgumentException(
+        "Can't make delete for a replica region. Operate on the primary");
+    }
+    Delete delete = new Delete(getMetaKeyForRegion(regionInfo));
     delete.addFamily(getCatalogFamily(), ts);
     return delete;
   }
@@ -1321,14 +1345,14 @@ public class MetaTableAccessor {
     if (splitA != null) {
       put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
         .setFamily(HConstants.CATALOG_FAMILY).setQualifier(HConstants.SPLITA_QUALIFIER)
-        .setTimestamp(put.getTimestamp()).setType(Type.Put).setValue(RegionInfo.toByteArray(splitA))
-        .build());
+        .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put)
+        .setValue(RegionInfo.toByteArray(splitA)).build());
     }
     if (splitB != null) {
       put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
         .setFamily(HConstants.CATALOG_FAMILY).setQualifier(HConstants.SPLITB_QUALIFIER)
-        .setTimestamp(put.getTimestamp()).setType(Type.Put).setValue(RegionInfo.toByteArray(splitB))
-        .build());
+        .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put)
+        .setValue(RegionInfo.toByteArray(splitB)).build());
     }
     return put;
   }
@@ -1399,9 +1423,14 @@ public class MetaTableAccessor {
     }
   }
 
-  private static Put addRegionStateToPut(Put put, RegionState.State state) throws IOException {
+  /**
+   * Set the column value corresponding to this {@code replicaId}'s {@link RegionState} to the
+   * provided {@code state}. Mutates the provided {@link Put}.
+   */
+  private static Put addRegionStateToPut(Put put, int replicaId, RegionState.State state)
+    throws IOException {
     put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
-      .setFamily(HConstants.CATALOG_FAMILY).setQualifier(getRegionStateColumn())
+      .setFamily(HConstants.CATALOG_FAMILY).setQualifier(getRegionStateColumn(replicaId))
       .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put).setValue(Bytes.toBytes(state.name()))
       .build());
     return put;
@@ -1412,9 +1441,9 @@ public class MetaTableAccessor {
    */
   public static void updateRegionState(Connection connection, RegionInfo ri,
     RegionState.State state) throws IOException {
-    Put put = new Put(RegionReplicaUtil.getRegionInfoForDefaultReplica(ri).getRegionName());
-    MetaTableAccessor.putsToMetaTable(connection,
-      Collections.singletonList(addRegionStateToPut(put, state)));
+    final Put put = makePutFromRegionInfo(ri);
+    addRegionStateToPut(put, ri.getReplicaId(), state);
+    putsToMetaTable(connection, Collections.singletonList(put));
   }
 
   /**
@@ -1432,7 +1461,7 @@ public class MetaTableAccessor {
   public static void addSplitsToParent(Connection connection, RegionInfo regionInfo,
     RegionInfo splitA, RegionInfo splitB) throws IOException {
     try (Table meta = getMetaHTable(connection)) {
-      Put put = makePutFromRegionInfo(regionInfo, EnvironmentEdgeManager.currentTime());
+      Put put = makePutFromRegionInfo(regionInfo);
       addDaughtersToPut(put, splitA, splitB);
       meta.put(put);
       debugLogMutation(put);
@@ -1483,7 +1512,7 @@ public class MetaTableAccessor {
       if (RegionReplicaUtil.isDefaultReplica(regionInfo)) {
         Put put = makePutFromRegionInfo(regionInfo, ts);
         // New regions are added with initial state of CLOSED.
-        addRegionStateToPut(put, RegionState.State.CLOSED);
+        addRegionStateToPut(put, regionInfo.getReplicaId(), RegionState.State.CLOSED);
         // Add empty locations for region replicas so that number of replicas can be cached
         // whenever the primary region is looked up from meta
         for (int i = 1; i < regionReplication; i++) {
@@ -1509,8 +1538,8 @@ public class MetaTableAccessor {
       String qualifier = String.format(HConstants.MERGE_QUALIFIER_PREFIX_STR + "%04d", counter++);
       put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
         .setFamily(HConstants.CATALOG_FAMILY).setQualifier(Bytes.toBytes(qualifier))
-        .setTimestamp(put.getTimestamp()).setType(Type.Put).setValue(RegionInfo.toByteArray(ri))
-        .build());
+        .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put)
+        .setValue(RegionInfo.toByteArray(ri)).build());
     }
     return put;
   }
@@ -1527,7 +1556,7 @@ public class MetaTableAccessor {
   public static void mergeRegions(Connection connection, RegionInfo mergedRegion,
     Map<RegionInfo, Long> parentSeqNum, ServerName sn, int regionReplication) throws IOException {
     try (Table meta = getMetaHTable(connection)) {
-      long time = HConstants.LATEST_TIMESTAMP;
+      long time = EnvironmentEdgeManager.currentTime();
       List<Mutation> mutations = new ArrayList<>();
       List<RegionInfo> replicationParents = new ArrayList<>();
       for (Map.Entry<RegionInfo, Long> e : parentSeqNum.entrySet()) {
@@ -1548,7 +1577,7 @@ public class MetaTableAccessor {
       // default OFFLINE state. If Master gets restarted after this step, start up sequence of
       // master tries to assign this offline region. This is followed by re-assignments of the
       // merged region from resumed {@link MergeTableRegionsProcedure}
-      addRegionStateToPut(putOfMerged, RegionState.State.CLOSED);
+      addRegionStateToPut(putOfMerged, RegionInfo.DEFAULT_REPLICA_ID, RegionState.State.CLOSED);
       mutations.add(putOfMerged);
       // The merged is a new region, openSeqNum = 1 is fine. ServerName may be null
       // if crash after merge happened but before we got to here.. means in-memory
@@ -1606,8 +1635,8 @@ public class MetaTableAccessor {
       // default OFFLINE state. If Master gets restarted after this step, start up sequence of
       // master tries to assign these offline regions. This is followed by re-assignments of the
       // daughter regions from resumed {@link SplitTableRegionProcedure}
-      addRegionStateToPut(putA, RegionState.State.CLOSED);
-      addRegionStateToPut(putB, RegionState.State.CLOSED);
+      addRegionStateToPut(putA, RegionInfo.DEFAULT_REPLICA_ID, RegionState.State.CLOSED);
+      addRegionStateToPut(putB, RegionInfo.DEFAULT_REPLICA_ID, RegionState.State.CLOSED);
 
       addSequenceNum(putA, 1, splitA.getReplicaId()); // new regions, openSeqNum = 1 is fine.
       addSequenceNum(putB, 1, splitB.getReplicaId());
@@ -1820,7 +1849,7 @@ public class MetaTableAccessor {
     throws IOException {
     Delete delete = new Delete(mergeRegion.getRegionName());
     // NOTE: We are doing a new hbase:meta read here.
-    Cell[] cells = getRegionResult(connection, mergeRegion.getRegionName()).rawCells();
+    Cell[] cells = getRegionResult(connection, mergeRegion).rawCells();
     if (cells == null || cells.length == 0) {
       return;
     }
@@ -1852,7 +1881,7 @@ public class MetaTableAccessor {
   public static Put addRegionInfo(final Put p, final RegionInfo hri) throws IOException {
     p.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(p.getRow())
       .setFamily(getCatalogFamily()).setQualifier(HConstants.REGIONINFO_QUALIFIER)
-      .setTimestamp(p.getTimestamp()).setType(Type.Put)
+      .setTimestamp(p.getTimestamp()).setType(Cell.Type.Put)
       // Serialize the Default Replica HRI otherwise scan of hbase:meta
       // shows an info:regioninfo value with encoded name and region
       // name that differs from that of the hbase;meta row.
@@ -1872,8 +1901,8 @@ public class MetaTableAccessor {
         .setQualifier(getStartCodeColumn(replicaId)).setTimestamp(p.getTimestamp())
         .setType(Cell.Type.Put).setValue(Bytes.toBytes(sn.getStartcode())).build())
       .add(builder.clear().setRow(p.getRow()).setFamily(getCatalogFamily())
-        .setQualifier(getSeqNumColumn(replicaId)).setTimestamp(p.getTimestamp()).setType(Type.Put)
-        .setValue(Bytes.toBytes(openSeqNum)).build());
+        .setQualifier(getSeqNumColumn(replicaId)).setTimestamp(p.getTimestamp())
+        .setType(Cell.Type.Put).setValue(Bytes.toBytes(openSeqNum)).build());
   }
 
   private static void writeRegionName(ByteArrayOutputStream out, byte[] regionName) {
@@ -1922,7 +1951,7 @@ public class MetaTableAccessor {
     byte[] value = getParentsBytes(parents);
     put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
       .setFamily(HConstants.REPLICATION_BARRIER_FAMILY).setQualifier(REPLICATION_PARENT_QUALIFIER)
-      .setTimestamp(put.getTimestamp()).setType(Type.Put).setValue(value).build());
+      .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put).setValue(value).build());
   }
 
   public static Put makePutForReplicationBarrier(RegionInfo regionInfo, long openSeqNum, long ts)
@@ -1938,7 +1967,7 @@ public class MetaTableAccessor {
   public static void addReplicationBarrier(Put put, long openSeqNum) throws IOException {
     put.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(put.getRow())
       .setFamily(HConstants.REPLICATION_BARRIER_FAMILY).setQualifier(HConstants.SEQNUM_QUALIFIER)
-      .setTimestamp(put.getTimestamp()).setType(Type.Put).setValue(Bytes.toBytes(openSeqNum))
+      .setTimestamp(put.getTimestamp()).setType(Cell.Type.Put).setValue(Bytes.toBytes(openSeqNum))
       .build());
   }
 
@@ -1946,8 +1975,8 @@ public class MetaTableAccessor {
     CellBuilder builder = CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY);
     return p
       .add(builder.clear().setRow(p.getRow()).setFamily(getCatalogFamily())
-        .setQualifier(getServerColumn(replicaId)).setTimestamp(p.getTimestamp()).setType(Type.Put)
-        .build())
+        .setQualifier(getServerColumn(replicaId)).setTimestamp(p.getTimestamp())
+        .setType(Cell.Type.Put).build())
       .add(builder.clear().setRow(p.getRow()).setFamily(getCatalogFamily())
         .setQualifier(getStartCodeColumn(replicaId)).setTimestamp(p.getTimestamp())
         .setType(Cell.Type.Put).build())
@@ -2096,7 +2125,7 @@ public class MetaTableAccessor {
   private static Put addSequenceNum(Put p, long openSeqNum, int replicaId) throws IOException {
     return p.add(CellBuilderFactory.create(CellBuilderType.SHALLOW_COPY).setRow(p.getRow())
       .setFamily(HConstants.CATALOG_FAMILY).setQualifier(getSeqNumColumn(replicaId))
-      .setTimestamp(p.getTimestamp()).setType(Type.Put).setValue(Bytes.toBytes(openSeqNum))
+      .setTimestamp(p.getTimestamp()).setType(Cell.Type.Put).setValue(Bytes.toBytes(openSeqNum))
       .build());
   }
 }
