@@ -106,6 +106,14 @@ public interface AsyncAdmin {
    */
   CompletableFuture<List<TableDescriptor>> listTableDescriptorsByNamespace(String name);
 
+  /**
+   * List all enabled or disabled table descriptors
+   * @param isEnabled is true means return enabled table descriptors, false means return disabled
+   *                  table descriptors
+   * @return a list of table names wrapped by a {@link CompletableFuture}.
+   */
+  CompletableFuture<List<TableDescriptor>> listTableDescriptorsByState(boolean isEnabled);
+
   /**
    * List all of the names of userspace tables.
    * @return a list of table names wrapped by a {@link CompletableFuture}.
@@ -137,6 +145,14 @@ public interface AsyncAdmin {
    */
   CompletableFuture<List<TableName>> listTableNamesByNamespace(String name);
 
+  /**
+   * List all enabled or disabled table names
+   * @param isEnabled is true means return enabled table names, false means return disabled table
+   *                  names
+   * @return a list of table names wrapped by a {@link CompletableFuture}.
+   */
+  CompletableFuture<List<TableName>> listTableNamesByState(boolean isEnabled);
+
   /**
    * Method for getting the tableDescriptor
    * @param tableName as a {@link TableName}
@@ -351,6 +367,14 @@ public interface AsyncAdmin {
    */
   CompletableFuture<Void> flush(TableName tableName, byte[] columnFamily);
 
+  /**
+   * Flush the specified column family stores on all regions of the passed table. This runs as a
+   * synchronous operation.
+   * @param tableName      table to flush
+   * @param columnFamilies column families within a table
+   */
+  CompletableFuture<Void> flush(TableName tableName, List<byte[]> columnFamilies);
+
   /**
    * Flush an individual region.
    * @param regionName region to flush
@@ -614,6 +638,12 @@ public interface AsyncAdmin {
    */
   CompletableFuture<Void> splitRegion(byte[] regionName, byte[] splitPoint);
 
+  /**
+   * Truncate an individual region.
+   * @param regionName region to truncate
+   */
+  CompletableFuture<Void> truncateRegion(byte[] regionName);
+
   /**
    * Assign an individual region.
    * @param regionName Encoded or full name of region to assign.
@@ -786,6 +816,43 @@ public interface AsyncAdmin {
    */
   CompletableFuture<Void> disableTableReplication(TableName tableName);
 
+  /**
+   * Check if a replication peer is enabled.
+   * @param peerId id of replication peer to check
+   * @return true if replication peer is enabled. The return value will be wrapped by a
+   *         {@link CompletableFuture}
+   */
+  CompletableFuture<Boolean> isReplicationPeerEnabled(String peerId);
+
+  /**
+   * Enable or disable replication peer modification.
+   * <p/>
+   * This is especially useful when you want to change the replication peer storage.
+   * @param on {@code true} means enable, otherwise disable
+   * @return the previous enable/disable state wrapped by a {@link CompletableFuture}
+   */
+  default CompletableFuture<Boolean> replicationPeerModificationSwitch(boolean on) {
+    return replicationPeerModificationSwitch(on, false);
+  }
+
+  /**
+   * Enable or disable replication peer modification.
+   * <p/>
+   * This is especially useful when you want to change the replication peer storage.
+   * @param on              {@code true} means enable, otherwise disable
+   * @param drainProcedures if {@code true}, will wait until all the running replication peer
+   *                        modification procedures finish
+   * @return the previous enable/disable state wrapped by a {@link CompletableFuture}
+   */
+  CompletableFuture<Boolean> replicationPeerModificationSwitch(boolean on, boolean drainProcedures);
+
+  /**
+   * Check whether replication peer modification is enabled.
+   * @return {@code true} if modification is enabled, otherwise {@code false}, wrapped by a
+   *         {@link CompletableFuture}
+   */
+  CompletableFuture<Boolean> isReplicationPeerModificationEnabled();
+
   /**
    * Take a snapshot for the given table. If the table is enabled, a FLUSH-type snapshot will be
    * taken. If the table is disabled, an offline snapshot is taken. Snapshots are considered unique
@@ -1649,4 +1716,9 @@ public interface AsyncAdmin {
    * Flush master local region
    */
   CompletableFuture<Void> flushMasterStore();
+
+  /**
+   * Get the list of cached files
+   */
+  CompletableFuture<List<String>> getCachedFilesList(ServerName serverName);
 }
