@@ -19,6 +19,9 @@ package org.apache.hadoop.hbase.client;
 
 import static org.apache.hadoop.hbase.client.ConnectionUtils.retries2Attempts;
 
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.yetus.audience.InterfaceAudience;
@@ -50,6 +53,8 @@ abstract class AsyncTableBuilderBase<C extends ScanResultConsumerBase>
 
   protected int startLogErrorsCnt;
 
+  protected Map<String, byte[]> requestAttributes = Collections.emptyMap();
+
   AsyncTableBuilderBase(TableName tableName, AsyncConnectionConfiguration connConf) {
     this.tableName = tableName;
     this.operationTimeoutNs = tableName.isSystemTable()
@@ -121,4 +126,13 @@ abstract class AsyncTableBuilderBase<C extends ScanResultConsumerBase>
     this.startLogErrorsCnt = startLogErrorsCnt;
     return this;
   }
+
+  @Override
+  public AsyncTableBuilder<C> setRequestAttribute(String key, byte[] value) {
+    if (this.requestAttributes.isEmpty()) {
+      this.requestAttributes = new HashMap<>();
+    }
+    this.requestAttributes.put(key, value);
+    return this;
+  }
 }
