@@ -32,9 +32,11 @@ import io.opentelemetry.api.trace.Span;
 import io.opentelemetry.api.trace.StatusCode;
 import io.opentelemetry.context.Scope;
 import java.io.IOException;
+import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
@@ -118,6 +120,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
 
   private final int startLogErrorsCnt;
 
+  private final Map<String, byte[]> requestAttributes;
+
   RawAsyncTableImpl(AsyncConnectionImpl conn, Timer retryTimer, AsyncTableBuilderBase<?> builder) {
     this.conn = conn;
     this.retryTimer = retryTimer;
@@ -144,6 +148,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
       ? conn.connConf.getMetaScannerCaching()
       : conn.connConf.getScannerCaching();
     this.defaultScannerMaxResultSize = conn.connConf.getScannerMaxResultSize();
+    this.requestAttributes = builder.requestAttributes;
   }
 
   @Override
@@ -248,7 +253,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
       .operationTimeout(operationTimeoutNs, TimeUnit.NANOSECONDS)
       .pause(pauseNs, TimeUnit.NANOSECONDS)
       .pauseForServerOverloaded(pauseNsForServerOverloaded, TimeUnit.NANOSECONDS)
-      .maxAttempts(maxAttempts).startLogErrorsCnt(startLogErrorsCnt);
+      .maxAttempts(maxAttempts).setRequestAttributes(requestAttributes)
+      .startLogErrorsCnt(startLogErrorsCnt).setRequestAttributes(requestAttributes);
   }
 
   private <T, R extends OperationWithAttributes & Row> SingleRequestCallerBuilder<T>
@@ -390,7 +396,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         () -> RawAsyncTableImpl.this.<Boolean> newCaller(row, put.getPriority(), rpcTimeoutNs)
           .action((controller, loc, stub) -> RawAsyncTableImpl.mutate(controller, loc, stub, put,
             (rn, p) -> RequestConverter.buildMutateRequest(rn, row, family, qualifier, op, value,
-              null, timeRange, p, HConstants.NO_NONCE, HConstants.NO_NONCE),
+              null, timeRange, p, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
             (c, r) -> r.getProcessed()))
           .call(),
         supplier);
@@ -406,7 +412,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         () -> RawAsyncTableImpl.this.<Boolean> newCaller(row, delete.getPriority(), rpcTimeoutNs)
           .action((controller, loc, stub) -> RawAsyncTableImpl.mutate(controller, loc, stub, delete,
             (rn, d) -> RequestConverter.buildMutateRequest(rn, row, family, qualifier, op, value,
-              null, timeRange, d, HConstants.NO_NONCE, HConstants.NO_NONCE),
+              null, timeRange, d, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
             (c, r) -> r.getProcessed()))
           .call(),
         supplier);
@@ -424,7 +430,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         .action((controller, loc, stub) -> RawAsyncTableImpl.this.mutateRow(controller, loc, stub,
           mutations,
           (rn, rm) -> RequestConverter.buildMultiRequest(rn, row, family, qualifier, op, value,
-            null, timeRange, rm, HConstants.NO_NONCE, HConstants.NO_NONCE),
+            null, timeRange, rm, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
           CheckAndMutateResult::isSuccess))
         .call(), supplier);
     }
@@ -465,7 +471,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         () -> RawAsyncTableImpl.this.<Boolean> newCaller(row, put.getPriority(), rpcTimeoutNs)
           .action((controller, loc, stub) -> RawAsyncTableImpl.mutate(controller, loc, stub, put,
             (rn, p) -> RequestConverter.buildMutateRequest(rn, row, null, null, null, null, filter,
-              timeRange, p, HConstants.NO_NONCE, HConstants.NO_NONCE),
+              timeRange, p, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
             (c, r) -> r.getProcessed()))
           .call(),
         supplier);
@@ -480,7 +486,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         () -> RawAsyncTableImpl.this.<Boolean> newCaller(row, delete.getPriority(), rpcTimeoutNs)
           .action((controller, loc, stub) -> RawAsyncTableImpl.mutate(controller, loc, stub, delete,
             (rn, d) -> RequestConverter.buildMutateRequest(rn, row, null, null, null, null, filter,
-              timeRange, d, HConstants.NO_NONCE, HConstants.NO_NONCE),
+              timeRange, d, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
             (c, r) -> r.getProcessed()))
           .call(),
         supplier);
@@ -497,7 +503,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
         .action((controller, loc, stub) -> RawAsyncTableImpl.this.mutateRow(controller, loc, stub,
           mutations,
           (rn, rm) -> RequestConverter.buildMultiRequest(rn, row, null, null, null, null, filter,
-            timeRange, rm, HConstants.NO_NONCE, HConstants.NO_NONCE),
+            timeRange, rm, HConstants.NO_NONCE, HConstants.NO_NONCE, false),
           CheckAndMutateResult::isSuccess))
         .call(), supplier);
     }
@@ -532,7 +538,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
               (rn, m) -> RequestConverter.buildMutateRequest(rn, checkAndMutate.getRow(),
                 checkAndMutate.getFamily(), checkAndMutate.getQualifier(),
                 checkAndMutate.getCompareOp(), checkAndMutate.getValue(),
-                checkAndMutate.getFilter(), checkAndMutate.getTimeRange(), m, nonceGroup, nonce),
+                checkAndMutate.getFilter(), checkAndMutate.getTimeRange(), m, nonceGroup, nonce,
+                checkAndMutate.isQueryMetricsEnabled()),
               (c, r) -> ResponseConverter.getCheckAndMutateResult(r, c.cellScanner())))
           .call();
       } else if (checkAndMutate.getAction() instanceof RowMutations) {
@@ -548,7 +555,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
               (rn, rm) -> RequestConverter.buildMultiRequest(rn, checkAndMutate.getRow(),
                 checkAndMutate.getFamily(), checkAndMutate.getQualifier(),
                 checkAndMutate.getCompareOp(), checkAndMutate.getValue(),
-                checkAndMutate.getFilter(), checkAndMutate.getTimeRange(), rm, nonceGroup, nonce),
+                checkAndMutate.getFilter(), checkAndMutate.getTimeRange(), rm, nonceGroup, nonce,
+                checkAndMutate.isQueryMetricsEnabled()),
               resp -> resp))
           .call();
       } else {
@@ -646,7 +654,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
   public void scan(Scan scan, AdvancedScanResultConsumer consumer) {
     new AsyncClientScanner(setDefaultScanConfig(scan), consumer, tableName, conn, retryTimer,
       pauseNs, pauseNsForServerOverloaded, maxAttempts, scanTimeoutNs, readRpcTimeoutNs,
-      startLogErrorsCnt).start();
+      startLogErrorsCnt, requestAttributes).start();
   }
 
   private long resultSize2CacheSize(long maxResultSize) {
@@ -742,7 +750,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
       .operationTimeout(operationTimeoutNs, TimeUnit.NANOSECONDS)
       .rpcTimeout(rpcTimeoutNs, TimeUnit.NANOSECONDS).pause(pauseNs, TimeUnit.NANOSECONDS)
       .pauseForServerOverloaded(pauseNsForServerOverloaded, TimeUnit.NANOSECONDS)
-      .maxAttempts(maxAttempts).startLogErrorsCnt(startLogErrorsCnt).call();
+      .maxAttempts(maxAttempts).startLogErrorsCnt(startLogErrorsCnt)
+      .setRequestAttributes(requestAttributes).call();
   }
 
   @Override
@@ -770,6 +779,11 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
     return unit.convert(scanTimeoutNs, TimeUnit.NANOSECONDS);
   }
 
+  @Override
+  public Map<String, byte[]> getRequestAttributes() {
+    return requestAttributes;
+  }
+
   private <S, R> CompletableFuture<R> coprocessorService(Function<RpcChannel, S> stubMaker,
     ServiceCaller<S, R> callable, RegionInfo region, byte[] row) {
     RegionCoprocessorRpcChannelImpl channel = new RegionCoprocessorRpcChannelImpl(conn, tableName,
@@ -818,10 +832,55 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
     }
   }
 
+  private <S, R> void coprocessorServiceUntilComplete(Function<RpcChannel, S> stubMaker,
+    ServiceCaller<S, R> callable, PartialResultCoprocessorCallback<S, R> callback,
+    AtomicBoolean locateFinished, AtomicInteger unfinishedRequest, RegionInfo region, Span span) {
+    addListener(coprocessorService(stubMaker, callable, region, region.getStartKey()), (r, e) -> {
+      try (Scope ignored = span.makeCurrent()) {
+        if (e != null) {
+          callback.onRegionError(region, e);
+        } else {
+          callback.onRegionComplete(region, r);
+        }
+
+        ServiceCaller<S, R> updatedCallable;
+        if (e == null && r != null) {
+          updatedCallable = callback.getNextCallable(r, region);
+        } else {
+          updatedCallable = null;
+        }
+
+        // If updatedCallable is non-null, we will be sending another request, so no need to
+        // decrement unfinishedRequest (recall that && short-circuits).
+        // If updatedCallable is null, and unfinishedRequest decrements to 0, we're done with the
+        // requests for this coprocessor call.
+        if (
+          updatedCallable == null && unfinishedRequest.decrementAndGet() == 0
+            && locateFinished.get()
+        ) {
+          callback.onComplete();
+        } else if (updatedCallable != null) {
+          Duration waitInterval = callback.getWaitInterval(r, region);
+          LOG.trace("Coprocessor returned incomplete result. "
+            + "Sleeping for {} before making follow-up request.", waitInterval);
+          if (!waitInterval.isZero()) {
+            AsyncConnectionImpl.RETRY_TIMER.newTimeout(
+              (timeout) -> coprocessorServiceUntilComplete(stubMaker, updatedCallable, callback,
+                locateFinished, unfinishedRequest, region, span),
+              waitInterval.toMillis(), TimeUnit.MILLISECONDS);
+          } else {
+            coprocessorServiceUntilComplete(stubMaker, updatedCallable, callback, locateFinished,
+              unfinishedRequest, region, span);
+          }
+        }
+      }
+    });
+  }
+
   private <S, R> void onLocateComplete(Function<RpcChannel, S> stubMaker,
-    ServiceCaller<S, R> callable, CoprocessorCallback<R> callback, List<HRegionLocation> locs,
-    byte[] endKey, boolean endKeyInclusive, AtomicBoolean locateFinished,
-    AtomicInteger unfinishedRequest, HRegionLocation loc, Throwable error) {
+    ServiceCaller<S, R> callable, PartialResultCoprocessorCallback<S, R> callback, byte[] endKey,
+    boolean endKeyInclusive, AtomicBoolean locateFinished, AtomicInteger unfinishedRequest,
+    HRegionLocation loc, Throwable error) {
     final Span span = Span.current();
     if (error != null) {
       callback.onError(error);
@@ -837,23 +896,13 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
       addListener(conn.getLocator().getRegionLocation(tableName, region.getEndKey(),
         RegionLocateType.CURRENT, operationTimeoutNs), (l, e) -> {
           try (Scope ignored = span.makeCurrent()) {
-            onLocateComplete(stubMaker, callable, callback, locs, endKey, endKeyInclusive,
-              locateFinished, unfinishedRequest, l, e);
+            onLocateComplete(stubMaker, callable, callback, endKey, endKeyInclusive, locateFinished,
+              unfinishedRequest, l, e);
           }
         });
     }
-    addListener(coprocessorService(stubMaker, callable, region, region.getStartKey()), (r, e) -> {
-      try (Scope ignored = span.makeCurrent()) {
-        if (e != null) {
-          callback.onRegionError(region, e);
-        } else {
-          callback.onRegionComplete(region, r);
-        }
-        if (unfinishedRequest.decrementAndGet() == 0 && locateFinished.get()) {
-          callback.onComplete();
-        }
-      }
-    });
+    coprocessorServiceUntilComplete(stubMaker, callable, callback, locateFinished,
+      unfinishedRequest, region, span);
   }
 
   private final class CoprocessorServiceBuilderImpl<S, R>
@@ -863,7 +912,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
 
     private final ServiceCaller<S, R> callable;
 
-    private final CoprocessorCallback<R> callback;
+    private final PartialResultCoprocessorCallback<S, R> callback;
 
     private byte[] startKey = HConstants.EMPTY_START_ROW;
 
@@ -874,7 +923,7 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
     private boolean endKeyInclusive;
 
     public CoprocessorServiceBuilderImpl(Function<RpcChannel, S> stubMaker,
-      ServiceCaller<S, R> callable, CoprocessorCallback<R> callback) {
+      ServiceCaller<S, R> callable, PartialResultCoprocessorCallback<S, R> callback) {
       this.stubMaker = Preconditions.checkNotNull(stubMaker, "stubMaker is null");
       this.callable = Preconditions.checkNotNull(callable, "callable is null");
       this.callback = Preconditions.checkNotNull(callback, "callback is null");
@@ -911,8 +960,8 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
           .getRegionLocation(tableName, startKey, regionLocateType, operationTimeoutNs);
         addListener(future, (loc, error) -> {
           try (Scope ignored1 = span.makeCurrent()) {
-            onLocateComplete(stubMaker, callable, callback, new ArrayList<>(), endKey,
-              endKeyInclusive, new AtomicBoolean(false), new AtomicInteger(0), loc, error);
+            onLocateComplete(stubMaker, callable, callback, endKey, endKeyInclusive,
+              new AtomicBoolean(false), new AtomicInteger(0), loc, error);
           }
         });
       }
@@ -923,6 +972,14 @@ class RawAsyncTableImpl implements AsyncTable<AdvancedScanResultConsumer> {
   public <S, R> CoprocessorServiceBuilder<S, R> coprocessorService(
     Function<RpcChannel, S> stubMaker, ServiceCaller<S, R> callable,
     CoprocessorCallback<R> callback) {
+    return new CoprocessorServiceBuilderImpl<>(stubMaker, callable,
+      new NoopPartialResultCoprocessorCallback<>(callback));
+  }
+
+  @Override
+  public <S, R> CoprocessorServiceBuilder<S, R> coprocessorService(
+    Function<RpcChannel, S> stubMaker, ServiceCaller<S, R> callable,
+    PartialResultCoprocessorCallback<S, R> callback) {
     return new CoprocessorServiceBuilderImpl<>(stubMaker, callable, callback);
   }
 }
