@@ -152,7 +152,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   long getRegionId();
 
   /**
-   * @return the regionName as an array of bytes.
+   * Returns the regionName as an array of bytes.
    * @see #getRegionNameAsString()
    */
   byte[] getRegionName();
@@ -182,7 +182,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   boolean isSplit();
 
   /**
-   * @return True if this region is offline.
+   * Returns True if this region is offline.
    * @deprecated since 3.0.0 and will be removed in 4.0.0
    * @see <a href="https://issues.apache.org/jira/browse/HBASE-25210">HBASE-25210</a>
    */
@@ -190,7 +190,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   boolean isOffline();
 
   /**
-   * @return True if this is a split parent region.
+   * Returns True if this is a split parent region.
    * @deprecated since 3.0.0 and will be removed in 4.0.0, Use {@link #isSplit()} instead.
    * @see <a href="https://issues.apache.org/jira/browse/HBASE-25210">HBASE-25210</a>
    */
@@ -201,9 +201,9 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   boolean isMetaRegion();
 
   /**
-   * @return true if the given inclusive range of rows is fully contained by this region. For
-   *         example, if the region is foo,a,g and this is passed ["b","c"] or ["a","c"] it will
-   *         return true, but if this is passed ["b","z"] it will return false.
+   * Returns true if the given inclusive range of rows is fully contained by this region. For
+   * example, if the region is foo,a,g and this is passed ["b","c"] or ["a","c"] it will return
+   * true, but if this is passed ["b","z"] it will return false.
    * @throws IllegalArgumentException if the range passed is invalid (ie. end &lt; start)
    */
   boolean containsRange(byte[] rangeStartKey, byte[] rangeEndKey);
@@ -263,16 +263,16 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @return Return a String of short, printable names for <code>hris</code> (usually encoded name)
-   *         for us logging.
+   * Returns a String of short, printable names for <code>hris</code> (usually encoded name) for us
+   * logging.
    */
   static String getShortNameToLog(RegionInfo... hris) {
     return getShortNameToLog(Arrays.asList(hris));
   }
 
   /**
-   * @return Return a String of short, printable names for <code>hris</code> (usually encoded name)
-   *         for us logging.
+   * Returns a String of short, printable names for <code>hris</code> (usually encoded name) for us
+   * logging.
    */
   static String getShortNameToLog(final List<RegionInfo> ris) {
     return ris.stream().map(RegionInfo::getEncodedName).collect(Collectors.toList()).toString();
@@ -317,9 +317,25 @@ public interface RegionInfo extends Comparable<RegionInfo> {
    * @return True if <code>regionName</code> represents an encoded name.
    */
   @InterfaceAudience.Private // For use by internals only.
-  public static boolean isEncodedRegionName(byte[] regionName) {
+  static boolean isEncodedRegionName(byte[] regionName) {
     // If not parseable as region name, presume encoded. TODO: add stringency; e.g. if hex.
-    return parseRegionNameOrReturnNull(regionName) == null && regionName.length <= MD5_HEX_LENGTH;
+    if (parseRegionNameOrReturnNull(regionName) == null) {
+      if (regionName.length > MD5_HEX_LENGTH) {
+        return false;
+      } else if (regionName.length == MD5_HEX_LENGTH) {
+        return true;
+      } else {
+        String encodedName = Bytes.toString(regionName);
+        try {
+          Integer.parseInt(encodedName);
+          // If this is a valid integer, it could be hbase:meta's encoded region name.
+          return true;
+        } catch (NumberFormatException er) {
+          return false;
+        }
+      }
+    }
+    return false;
   }
 
   /**
@@ -345,8 +361,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @param bytes A pb RegionInfo serialized with a pb magic prefix.
-   * @return A deserialized {@link RegionInfo}
+   * Returns A deserialized {@link RegionInfo}
    */
   @InterfaceAudience.Private
   static RegionInfo parseFrom(final byte[] bytes) throws DeserializationException {
@@ -355,6 +370,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
+   * Parse a serialized representation of {@link RegionInfo}
    * @param bytes  A pb RegionInfo serialized with a pb magic prefix.
    * @param offset starting point in the byte array
    * @param len    length to read on the byte array
@@ -413,7 +429,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @return This instance serialized as protobuf w/ a magic pb prefix.
+   * Returns This instance serialized as protobuf w/ a magic pb prefix.
    * @see #parseFrom(byte[])
    */
   static byte[] toByteArray(RegionInfo ri) {
@@ -741,7 +757,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @return True if region is next, adjacent but 'after' this one.
+   * Returns True if region is next, adjacent but 'after' this one.
    * @see #isAdjacent(RegionInfo)
    * @see #areAdjacent(RegionInfo, RegionInfo)
    */
@@ -750,7 +766,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @return True if region is adjacent, either just before or just after this one.
+   * Returns True if region is adjacent, either just before or just after this one.
    * @see #isNext(RegionInfo)
    */
   default boolean isAdjacent(RegionInfo other) {
@@ -763,7 +779,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
   }
 
   /**
-   * @return True if an overlap in region range.
+   * Returns True if an overlap in region range.
    * @see #isDegenerate()
    */
   default boolean isOverlap(RegionInfo other) {
@@ -789,6 +805,7 @@ public interface RegionInfo extends Comparable<RegionInfo> {
     return Bytes.compareTo(getStartKey(), other.getEndKey()) < 0;
   }
 
+  @Override
   default int compareTo(RegionInfo other) {
     return RegionInfo.COMPARATOR.compare(this, other);
   }
