@@ -97,6 +97,7 @@ public class Result implements CellScannable, CellScanner {
    */
   private int cellScannerIndex = INITIAL_CELLSCANNER_INDEX;
   private RegionLoadStats stats;
+  private QueryMetrics metrics = null;
 
   private final boolean readonly;
 
@@ -791,9 +792,7 @@ public class Result implements CellScannable, CellScanner {
       }
       prevRow = currentRow;
       stale = stale || r.isStale();
-      for (Cell c : r.rawCells()) {
-        cells.add(c);
-      }
+      Collections.addAll(cells, r.rawCells());
     }
 
     return Result.create(cells, null, stale);
@@ -837,9 +836,7 @@ public class Result implements CellScannable, CellScanner {
   public Cell current() {
     if (
       isEmpty() || cellScannerIndex == INITIAL_CELLSCANNER_INDEX || cellScannerIndex >= cells.length
-    ) {
-      return null;
-    }
+    ) return null;
     return this.cells[cellScannerIndex];
   }
 
@@ -907,6 +904,11 @@ public class Result implements CellScannable, CellScanner {
     this.stats = loadStats;
   }
 
+  @InterfaceAudience.Private
+  public void setMetrics(QueryMetrics metrics) {
+    this.metrics = metrics;
+  }
+
   /**
    * Returns the associated statistics about the region from which this was returned. Can be
    * <tt>null</tt> if stats are disabled.
@@ -915,6 +917,11 @@ public class Result implements CellScannable, CellScanner {
     return stats;
   }
 
+  /** Returns the query metrics, or {@code null} if we do not enable metrics. */
+  public QueryMetrics getMetrics() {
+    return metrics;
+  }
+
   /**
    * All methods modifying state of Result object must call this method to ensure that special
    * purpose immutable Results can't be accidentally modified.
