@@ -26,10 +26,14 @@ import static org.apache.hadoop.hbase.ipc.IPCUtil.toIOE;
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.net.UnknownHostException;
+import java.util.Set;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
+import javax.security.sasl.SaslException;
+import org.apache.hadoop.hbase.client.ConnectionUtils;
+import org.apache.hadoop.hbase.io.crypto.tls.X509Util;
 import org.apache.hadoop.hbase.ipc.BufferCallBeforeInitHandler.BufferCallEvent;
 import org.apache.hadoop.hbase.ipc.HBaseRpcController.CancellationCallback;
 import org.apache.hadoop.hbase.security.NettyHBaseRpcConnectionHeaderHandler;
@@ -56,6 +60,8 @@ import org.apache.hbase.thirdparty.io.netty.channel.ChannelOption;
 import org.apache.hbase.thirdparty.io.netty.channel.ChannelPipeline;
 import org.apache.hbase.thirdparty.io.netty.channel.EventLoop;
 import org.apache.hbase.thirdparty.io.netty.handler.codec.LengthFieldBasedFrameDecoder;
+import org.apache.hbase.thirdparty.io.netty.handler.ssl.SslContext;
+import org.apache.hbase.thirdparty.io.netty.handler.ssl.SslHandler;
 import org.apache.hbase.thirdparty.io.netty.handler.timeout.IdleStateHandler;
 import org.apache.hbase.thirdparty.io.netty.handler.timeout.ReadTimeoutHandler;
 import org.apache.hbase.thirdparty.io.netty.util.ReferenceCountUtil;
@@ -101,7 +107,7 @@ class NettyRpcConnection extends RpcConnection {
   NettyRpcConnection(NettyRpcClient rpcClient, ConnectionId remoteId) throws IOException {
     super(rpcClient.conf, AbstractRpcClient.WHEEL_TIMER, remoteId, rpcClient.clusterId,
       rpcClient.userProvider.isHBaseSecurityEnabled(), rpcClient.codec, rpcClient.compressor,
-      rpcClient.metrics);
+      rpcClient.cellBlockBuilder, rpcClient.metrics, rpcClient.connectionAttributes);
     this.rpcClient = rpcClient;
     this.eventLoop = rpcClient.group.next();
     byte[] connectionHeaderPreamble = getConnectionHeaderPreamble();
@@ -154,7 +160,7 @@ class NettyRpcConnection extends RpcConnection {
     });
   }
 
-  private void established(Channel ch) throws IOException {
+  private void established(Channel ch) {
     assert eventLoop.inEventLoop();
     ch.pipeline()
       .addBefore(BufferCallBeforeInitHandler.NAME, null,
@@ -166,6 +172,11 @@ class NettyRpcConnection extends RpcConnection {
       .fireUserEventTriggered(BufferCallEvent.success());
   }
 
+  private void saslEstablished(Channel ch, String serverPrincipal) {
+    saslNegotiationDone(serverPrincipal, true);
+    established(ch);
+  }
+
   private boolean reloginInProgress;
 
   @SuppressWarnings("FutureReturnValueIgnored")
@@ -199,36 +210,42 @@ class NettyRpcConnection extends RpcConnection {
     // fail all pending calls
     ch.pipeline().fireUserEventTriggered(BufferCallEvent.fail(e));
     shutdown0();
+    rpcClient.failedServers.addToFailedServers(remoteId.getAddress(), e);
   }
 
-  private void saslNegotiate(final Channel ch) {
+  private void saslFailInit(Channel ch, String serverPrincipal, IOException error) {
     assert eventLoop.inEventLoop();
+    saslNegotiationDone(serverPrincipal, false);
+    failInit(ch, error);
+  }
+
+  private void saslNegotiate(Channel ch, String serverPrincipal) {
+    assert eventLoop.inEventLoop();
+    NettyFutureUtils.safeWriteAndFlush(ch, connectionHeaderPreamble.retainedDuplicate());
     UserGroupInformation ticket = provider.getRealUser(remoteId.getTicket());
     if (ticket == null) {
-      failInit(ch, new FatalConnectionException("ticket/user is null"));
+      saslFailInit(ch, serverPrincipal, new FatalConnectionException("ticket/user is null"));
       return;
     }
     Promise<Boolean> saslPromise = ch.eventLoop().newPromise();
     final NettyHBaseSaslRpcClientHandler saslHandler;
     try {
       saslHandler = new NettyHBaseSaslRpcClientHandler(saslPromise, ticket, provider, token,
-        ((InetSocketAddress) ch.remoteAddress()).getAddress(), securityInfo,
+        ((InetSocketAddress) ch.remoteAddress()).getAddress(), serverPrincipal,
         rpcClient.fallbackAllowed, this.rpcClient.conf);
     } catch (IOException e) {
-      failInit(ch, e);
+      saslFailInit(ch, serverPrincipal, e);
       return;
     }
     ch.pipeline().addBefore(BufferCallBeforeInitHandler.NAME, null, new SaslChallengeDecoder())
-      .addBefore(BufferCallBeforeInitHandler.NAME, null, saslHandler);
+      .addBefore(BufferCallBeforeInitHandler.NAME, NettyHBaseSaslRpcClientHandler.HANDLER_NAME,
+        saslHandler);
     NettyFutureUtils.addListener(saslPromise, new FutureListener<Boolean>() {
 
       @Override
       public void operationComplete(Future<Boolean> future) throws Exception {
         if (future.isSuccess()) {
           ChannelPipeline p = ch.pipeline();
-          p.remove(SaslChallengeDecoder.class);
-          p.remove(NettyHBaseSaslRpcClientHandler.class);
-
           // check if negotiate with server for connection header is necessary
           if (saslHandler.isNeedProcessConnectionHeader()) {
             Promise<Boolean> connectionHeaderPromise = ch.eventLoop().newPromise();
@@ -252,29 +269,99 @@ class NettyRpcConnection extends RpcConnection {
                   p.remove(NettyHBaseRpcConnectionHeaderHandler.class);
                   // don't send connection header, NettyHBaseRpcConnectionHeaderHandler
                   // sent it already
-                  established(ch);
+                  saslEstablished(ch, serverPrincipal);
                 } else {
                   final Throwable error = future.cause();
                   scheduleRelogin(error);
-                  failInit(ch, toIOE(error));
+                  saslFailInit(ch, serverPrincipal, toIOE(error));
                 }
               }
             });
           } else {
             // send the connection header to server
-            NettyFutureUtils.safeWrite(ch, connectionHeaderWithLength.retainedDuplicate());
-            established(ch);
+            ch.write(connectionHeaderWithLength.retainedDuplicate());
+            saslEstablished(ch, serverPrincipal);
           }
         } else {
           final Throwable error = future.cause();
           scheduleRelogin(error);
-          failInit(ch, toIOE(error));
+          saslFailInit(ch, serverPrincipal, toIOE(error));
         }
       }
     });
   }
 
-  private void connect() throws UnknownHostException {
+  private void getConnectionRegistry(Channel ch, Call connectionRegistryCall) {
+    assert eventLoop.inEventLoop();
+    PreambleCallHandler.setup(ch.pipeline(), rpcClient.readTO, this,
+      RpcClient.REGISTRY_PREAMBLE_HEADER, connectionRegistryCall);
+  }
+
+  private void onSecurityPreambleError(Channel ch, Set<String> serverPrincipals,
+    IOException error) {
+    assert eventLoop.inEventLoop();
+    LOG.debug("Error when trying to do a security preamble call to {}", remoteId.address, error);
+    if (ConnectionUtils.isUnexpectedPreambleHeaderException(error)) {
+      // this means we are connecting to an old server which does not support the security
+      // preamble call, so we should fallback to randomly select a principal to use
+      // TODO: find a way to reconnect without failing all the pending calls, for now, when we
+      // reach here, shutdown should have already been scheduled
+      return;
+    }
+    if (IPCUtil.isSecurityNotEnabledException(error)) {
+      // server tells us security is not enabled, then we should check whether fallback to
+      // simple is allowed, if so we just go without security, otherwise we should fail the
+      // negotiation immediately
+      if (rpcClient.fallbackAllowed) {
+        // TODO: just change the preamble and skip the fallback to simple logic, for now, just
+        // select the first principal can finish the connection setup, but waste one client
+        // message
+        saslNegotiate(ch, serverPrincipals.iterator().next());
+      } else {
+        failInit(ch, new FallbackDisallowedException());
+      }
+      return;
+    }
+    // usually we should not reach here, but for robust, just randomly select a principal to
+    // connect
+    saslNegotiate(ch, randomSelect(serverPrincipals));
+  }
+
+  private void onSecurityPreambleFinish(Channel ch, Set<String> serverPrincipals,
+    Call securityPreambleCall) {
+    assert eventLoop.inEventLoop();
+    String serverPrincipal;
+    try {
+      serverPrincipal = chooseServerPrincipal(serverPrincipals, securityPreambleCall);
+    } catch (SaslException e) {
+      failInit(ch, e);
+      return;
+    }
+    saslNegotiate(ch, serverPrincipal);
+  }
+
+  private void saslNegotiate(Channel ch) throws IOException {
+    assert eventLoop.inEventLoop();
+    Set<String> serverPrincipals = getServerPrincipals();
+    if (serverPrincipals.size() == 1) {
+      saslNegotiate(ch, serverPrincipals.iterator().next());
+      return;
+    }
+    // this means we use kerberos authentication and there are multiple server principal candidates,
+    // in this way we need to send a special preamble header to get server principal from server
+    Call securityPreambleCall = createSecurityPreambleCall(call -> {
+      assert eventLoop.inEventLoop();
+      if (call.error != null) {
+        onSecurityPreambleError(ch, serverPrincipals, call.error);
+      } else {
+        onSecurityPreambleFinish(ch, serverPrincipals, call);
+      }
+    });
+    PreambleCallHandler.setup(ch.pipeline(), rpcClient.readTO, this,
+      RpcClient.SECURITY_PREAMBLE_HEADER, securityPreambleCall);
+  }
+
+  private void connect(Call connectionRegistryCall) throws UnknownHostException {
     assert eventLoop.inEventLoop();
     LOG.trace("Connecting to {}", remoteId.getAddress());
     InetSocketAddress remoteAddr = getRemoteInetAddress(rpcClient.metrics);
@@ -283,33 +370,69 @@ class NettyRpcConnection extends RpcConnection {
       .option(ChannelOption.SO_KEEPALIVE, rpcClient.tcpKeepAlive)
       .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, rpcClient.connectTO)
       .handler(new ChannelInitializer<Channel>() {
-
         @Override
         protected void initChannel(Channel ch) throws Exception {
+          if (conf.getBoolean(X509Util.HBASE_CLIENT_NETTY_TLS_ENABLED, false)) {
+            SslContext sslContext = rpcClient.getSslContext();
+            SslHandler sslHandler = sslContext.newHandler(ch.alloc(),
+              remoteId.address.getHostName(), remoteId.address.getPort());
+            sslHandler.setHandshakeTimeoutMillis(
+              conf.getInt(X509Util.HBASE_CLIENT_NETTY_TLS_HANDSHAKETIMEOUT,
+                X509Util.DEFAULT_HANDSHAKE_DETECTION_TIMEOUT_MILLIS));
+            ch.pipeline().addFirst(sslHandler);
+            LOG.debug("SSL handler added with handshake timeout {} ms",
+              sslHandler.getHandshakeTimeoutMillis());
+          }
           ch.pipeline().addLast(BufferCallBeforeInitHandler.NAME,
             new BufferCallBeforeInitHandler());
         }
       }).localAddress(rpcClient.localAddr).remoteAddress(remoteAddr).connect()
       .addListener(new ChannelFutureListener() {
 
+        private void succeed(Channel ch) throws IOException {
+          if (connectionRegistryCall != null) {
+            getConnectionRegistry(ch, connectionRegistryCall);
+            return;
+          }
+          if (!useSasl) {
+            // BufferCallBeforeInitHandler will call ctx.flush when receiving the
+            // BufferCallEvent.success() event, so here we just use write for the below two messages
+            NettyFutureUtils.safeWrite(ch, connectionHeaderPreamble.retainedDuplicate());
+            NettyFutureUtils.safeWrite(ch, connectionHeaderWithLength.retainedDuplicate());
+            established(ch);
+          } else {
+            saslNegotiate(ch);
+          }
+        }
+
+        private void fail(Channel ch, Throwable error) {
+          IOException ex = toIOE(error);
+          LOG.warn("Exception encountered while connecting to the server " + remoteId.getAddress(),
+            ex);
+          if (connectionRegistryCall != null) {
+            connectionRegistryCall.setException(ex);
+          }
+          failInit(ch, ex);
+        }
+
         @Override
         public void operationComplete(ChannelFuture future) throws Exception {
           Channel ch = future.channel();
           if (!future.isSuccess()) {
-            IOException ex = toIOE(future.cause());
-            LOG.warn(
-              "Exception encountered while connecting to the server " + remoteId.getAddress(), ex);
-            failInit(ch, ex);
-            rpcClient.failedServers.addToFailedServers(remoteId.getAddress(), future.cause());
+            fail(ch, future.cause());
             return;
           }
-          NettyFutureUtils.safeWriteAndFlush(ch, connectionHeaderPreamble.retainedDuplicate());
-          if (useSasl) {
-            saslNegotiate(ch);
+          SslHandler sslHandler = ch.pipeline().get(SslHandler.class);
+          if (sslHandler != null) {
+            NettyFutureUtils.addListener(sslHandler.handshakeFuture(), f -> {
+              if (f.isSuccess()) {
+                succeed(ch);
+              } else {
+                fail(ch, f.cause());
+              }
+            });
           } else {
-            // send the connection header to server
-            NettyFutureUtils.safeWrite(ch, connectionHeaderWithLength.retainedDuplicate());
-            established(ch);
+            succeed(ch);
           }
         }
       }).channel();
@@ -317,6 +440,12 @@ class NettyRpcConnection extends RpcConnection {
 
   private void sendRequest0(Call call, HBaseRpcController hrc) throws IOException {
     assert eventLoop.inEventLoop();
+    if (call.isConnectionRegistryCall()) {
+      // For get connection registry call, we will send a special preamble header to get the
+      // response, instead of sending a real rpc call. See HBASE-25051
+      connect(call);
+      return;
+    }
     if (reloginInProgress) {
       throw new IOException(RpcConnectionConstants.RELOGIN_IS_IN_PROGRESS);
     }
@@ -337,7 +466,7 @@ class NettyRpcConnection extends RpcConnection {
           setCancelled(call);
         } else {
           if (channel == null) {
-            connect();
+            connect(null);
           }
           scheduleTimeoutTask(call);
           NettyFutureUtils.addListener(channel.writeAndFlush(call), new ChannelFutureListener() {
