@@ -27,6 +27,7 @@ import java.util.List;
 import java.util.UUID;
 import java.util.regex.Pattern;
 import org.apache.commons.lang3.ArrayUtils;
+import org.apache.hadoop.hbase.conf.ConfigKey;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.yetus.audience.InterfaceAudience;
 
@@ -351,7 +352,8 @@ public final class HConstants {
   public static final int DEFAULT_VERSION_FILE_WRITE_ATTEMPTS = 3;
 
   /** Parameter name and default value for how often a region should perform a major compaction */
-  public static final String MAJOR_COMPACTION_PERIOD = "hbase.hregion.majorcompaction";
+  public static final String MAJOR_COMPACTION_PERIOD =
+    ConfigKey.LONG("hbase.hregion.majorcompaction");
   public static final long DEFAULT_MAJOR_COMPACTION_PERIOD = 1000 * 60 * 60 * 24 * 7; // 7 days
 
   /**
@@ -360,16 +362,17 @@ public final class HConstants {
    * either side of {@link HConstants#MAJOR_COMPACTION_PERIOD}. Default to 0.5 so jitter has us fall
    * evenly either side of when the compaction should run.
    */
-  public static final String MAJOR_COMPACTION_JITTER = "hbase.hregion.majorcompaction.jitter";
+  public static final String MAJOR_COMPACTION_JITTER =
+    ConfigKey.FLOAT("hbase.hregion.majorcompaction.jitter");
   public static final float DEFAULT_MAJOR_COMPACTION_JITTER = 0.50F;
 
   /** Parameter name for the maximum batch of KVs to be used in flushes and compactions */
-  public static final String COMPACTION_KV_MAX = "hbase.hstore.compaction.kv.max";
+  public static final String COMPACTION_KV_MAX = ConfigKey.INT("hbase.hstore.compaction.kv.max");
   public static final int COMPACTION_KV_MAX_DEFAULT = 10;
 
   /** Parameter name for the scanner size limit to be used in compactions */
   public static final String COMPACTION_SCANNER_SIZE_MAX =
-    "hbase.hstore.compaction.scanner.size.limit";
+    ConfigKey.LONG("hbase.hstore.compaction.scanner.size.limit");
   public static final long COMPACTION_SCANNER_SIZE_MAX_DEFAULT = 10 * 1024 * 1024L; // 10MB
 
   /** Parameter name for HBase instance root directory */
@@ -429,7 +432,7 @@ public final class HConstants {
   public static final String HREGION_COMPACTIONDIR_NAME = "compaction.dir";
 
   /** Conf key for the max file size after which we split the region */
-  public static final String HREGION_MAX_FILESIZE = "hbase.hregion.max.filesize";
+  public static final String HREGION_MAX_FILESIZE = ConfigKey.LONG("hbase.hregion.max.filesize");
 
   /** Default maximum file size */
   public static final long DEFAULT_MAX_FILE_SIZE = 10 * 1024 * 1024 * 1024L;
@@ -454,7 +457,7 @@ public final class HConstants {
    * The max number of threads used for opening and closing stores or store files in parallel
    */
   public static final String HSTORE_OPEN_AND_CLOSE_THREADS_MAX =
-    "hbase.hstore.open.and.close.threads.max";
+    ConfigKey.INT("hbase.hstore.open.and.close.threads.max");
 
   /**
    * The default number for the max number of threads used for opening and closing stores or store
@@ -468,7 +471,7 @@ public final class HConstants {
    * update traffic.
    */
   public static final String HREGION_MEMSTORE_BLOCK_MULTIPLIER =
-    "hbase.hregion.memstore.block.multiplier";
+    ConfigKey.INT("hbase.hregion.memstore.block.multiplier", v -> v > 0);
 
   /**
    * Default value for hbase.hregion.memstore.block.multiplier
@@ -476,7 +479,8 @@ public final class HConstants {
   public static final int DEFAULT_HREGION_MEMSTORE_BLOCK_MULTIPLIER = 4;
 
   /** Conf key for the memstore size at which we flush the memstore */
-  public static final String HREGION_MEMSTORE_FLUSH_SIZE = "hbase.hregion.memstore.flush.size";
+  public static final String HREGION_MEMSTORE_FLUSH_SIZE =
+    ConfigKey.LONG("hbase.hregion.memstore.flush.size", v -> v > 0);
 
   public static final String HREGION_EDITS_REPLAY_SKIP_ERRORS =
     "hbase.hregion.edits.replay.skip.errors";
@@ -493,7 +497,8 @@ public final class HConstants {
   public static final String CLUSTER_ID_DEFAULT = "default-cluster";
 
   /** Parameter name for # days to keep MVCC values during a major compaction */
-  public static final String KEEP_SEQID_PERIOD = "hbase.hstore.compaction.keep.seqId.period";
+  public static final String KEEP_SEQID_PERIOD =
+    ConfigKey.INT("hbase.hstore.compaction.keep.seqId.period");
   /** At least to keep MVCC values in hfiles for 5 days */
   public static final int MIN_KEEP_SEQID_PERIOD = 5;
 
