@@ -26,9 +26,9 @@ import java.lang.management.MonitorInfo;
 import java.lang.management.ThreadInfo;
 import java.lang.management.ThreadMXBean;
 import java.nio.charset.StandardCharsets;
-import java.time.Instant;
-import java.time.ZoneId;
-import java.time.format.DateTimeFormatter;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
+import java.util.Date;
 import java.util.Locale;
 import java.util.Map;
 import org.junit.runner.notification.Failure;
@@ -40,10 +40,7 @@ import org.junit.runner.notification.RunListener;
  */
 public class TimedOutTestsListener extends RunListener {
 
-  private static final String TEST_TIMED_OUT_PREFIX = "test timed out after";
-
-  private static final DateTimeFormatter TIMESTAMP_FORMATTER =
-    DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss,SSS Z").withZone(ZoneId.systemDefault());
+  static final String TEST_TIMED_OUT_PREFIX = "test timed out after";
 
   private static String INDENT = "    ";
 
@@ -70,11 +67,13 @@ public class TimedOutTestsListener extends RunListener {
     output.flush();
   }
 
+  @SuppressWarnings("JavaUtilDate")
   public static String buildThreadDiagnosticString() {
     StringWriter sw = new StringWriter();
     PrintWriter output = new PrintWriter(sw);
 
-    output.println(String.format("Timestamp: %s", TIMESTAMP_FORMATTER.format(Instant.now())));
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss,SSS");
+    output.println(String.format("Timestamp: %s", dateFormat.format(new Date())));
     output.println();
     output.println(buildThreadDump());
 
@@ -88,7 +87,7 @@ public class TimedOutTestsListener extends RunListener {
     return sw.toString();
   }
 
-  private static String buildThreadDump() {
+  static String buildThreadDump() {
     StringBuilder dump = new StringBuilder();
     Map<Thread, StackTraceElement[]> stackTraces = Thread.getAllStackTraces();
     for (Map.Entry<Thread, StackTraceElement[]> e : stackTraces.entrySet()) {
@@ -110,7 +109,7 @@ public class TimedOutTestsListener extends RunListener {
     return dump.toString();
   }
 
-  private static String buildDeadlockInfo() {
+  static String buildDeadlockInfo() {
     ThreadMXBean threadBean = ManagementFactory.getThreadMXBean();
     long[] threadIds = threadBean.findMonitorDeadlockedThreads();
     if (threadIds != null && threadIds.length > 0) {
