@@ -15,11 +15,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.hadoop.hbase.chaos.actions;
 
 import java.io.IOException;
 import java.util.Random;
+import java.util.concurrent.ThreadLocalRandom;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.io.compress.Compression.Algorithm;
 import org.apache.hadoop.io.compress.Compressor;
@@ -31,15 +31,14 @@ import org.slf4j.LoggerFactory;
  */
 public class ChangeCompressionAction extends Action {
   private final TableName tableName;
-  private final Random random;
   private static final Logger LOG = LoggerFactory.getLogger(ChangeCompressionAction.class);
 
   public ChangeCompressionAction(TableName tableName) {
     this.tableName = tableName;
-    this.random = new Random();
   }
 
-  @Override protected Logger getLogger() {
+  @Override
+  protected Logger getLogger() {
     return LOG;
   }
 
@@ -48,16 +47,15 @@ public class ChangeCompressionAction extends Action {
     // Possible compression algorithms. If an algorithm is not supported,
     // modifyTable will fail, so there is no harm.
     Algorithm[] possibleAlgos = Algorithm.values();
-
     // Since not every compression algorithm is supported,
     // let's use the same algorithm for all column families.
-
+    Random rand = ThreadLocalRandom.current();
     // If an unsupported compression algorithm is chosen, pick a different one.
     // This is to work around the issue that modifyTable() does not throw remote
     // exception.
     Algorithm algo;
     do {
-      algo = possibleAlgos[random.nextInt(possibleAlgos.length)];
+      algo = possibleAlgos[rand.nextInt(possibleAlgos.length)];
 
       try {
         Compressor c = algo.getCompressor();
@@ -66,8 +64,8 @@ public class ChangeCompressionAction extends Action {
         algo.returnCompressor(c);
         break;
       } catch (Throwable t) {
-        getLogger().info("Performing action: Changing compression algorithms to " + algo +
-            " is not supported, pick another one");
+        getLogger().info("Performing action: Changing compression algorithms to " + algo
+          + " is not supported, pick another one");
       }
     } while (true);
 
@@ -75,7 +73,7 @@ public class ChangeCompressionAction extends Action {
     getLogger().debug("Performing action: Changing compression algorithms on "
       + tableName.getNameAsString() + " to " + chosenAlgo);
     modifyAllTableColumns(tableName, columnFamilyDescriptorBuilder -> {
-      if (random.nextBoolean()) {
+      if (rand.nextBoolean()) {
         columnFamilyDescriptorBuilder.setCompactionCompressionType(chosenAlgo);
       } else {
         columnFamilyDescriptorBuilder.setCompressionType(chosenAlgo);
