@@ -346,6 +346,25 @@ public abstract class Procedure<TEnvironment> implements Comparable<Procedure<TE
     // no-op
   }
 
+  /**
+   * Called before we call the execute method of this procedure, but after we acquire the execution
+   * lock and procedure scheduler lock.
+   */
+  protected void beforeExec(TEnvironment env) throws ProcedureSuspendedException {
+    // no-op
+  }
+
+  /**
+   * Called after we call the execute method of this procedure, and also after we initialize all the
+   * sub procedures and persist the the state if persistence is needed.
+   * <p>
+   * This is for doing some hooks after we initialize the sub procedures. See HBASE-29259 for more
+   * details on why we can not release the region lock inside the execute method.
+   */
+  protected void afterExec(TEnvironment env) {
+    // no-op
+  }
+
   /**
    * Called when the procedure is marked as completed (success or rollback). The procedure
    * implementor may use this method to cleanup in-memory states. This operation will not be retried
