@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -30,17 +30,18 @@ import org.junit.experimental.categories.Category;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.MasterProcedureProtos.ServerCrashState;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.ProcedureProtos.ProcedureState;
 
-@Category({MasterTests.class, SmallTests.class})
+@Category({ MasterTests.class, SmallTests.class })
 public class TestProcedureToString {
 
   @ClassRule
   public static final HBaseClassTestRule CLASS_RULE =
-      HBaseClassTestRule.forClass(TestProcedureToString.class);
+    HBaseClassTestRule.forClass(TestProcedureToString.class);
 
   /**
    * A do-nothing environment for BasicProcedure.
    */
-  static class BasicProcedureEnv {};
+  static class BasicProcedureEnv {
+  };
 
   /**
    * A do-nothing basic procedure just for testing toString.
@@ -48,8 +49,8 @@ public class TestProcedureToString {
   static class BasicProcedure extends Procedure<BasicProcedureEnv> {
     @Override
     protected Procedure<BasicProcedureEnv>[] execute(BasicProcedureEnv env)
-        throws ProcedureYieldException, InterruptedException {
-      return new Procedure [] {this};
+      throws ProcedureYieldException, InterruptedException {
+      return new Procedure[] { this };
     }
 
     @Override
@@ -62,13 +63,11 @@ public class TestProcedureToString {
     }
 
     @Override
-    protected void serializeStateData(ProcedureStateSerializer serializer)
-        throws IOException {
+    protected void serializeStateData(ProcedureStateSerializer serializer) throws IOException {
     }
 
     @Override
-    protected void deserializeStateData(ProcedureStateSerializer serializer)
-        throws IOException {
+    protected void deserializeStateData(ProcedureStateSerializer serializer) throws IOException {
     }
   }
 
@@ -106,17 +105,17 @@ public class TestProcedureToString {
    * Do-nothing SimpleMachineProcedure for checking its toString.
    */
   static class SimpleStateMachineProcedure
-          extends StateMachineProcedure<BasicProcedureEnv, ServerCrashState> {
+    extends StateMachineProcedure<BasicProcedureEnv, ServerCrashState> {
     @Override
-    protected org.apache.hadoop.hbase.procedure2.StateMachineProcedure.Flow executeFromState(
-            BasicProcedureEnv env, ServerCrashState state)
-            throws ProcedureYieldException, InterruptedException {
+    protected org.apache.hadoop.hbase.procedure2.StateMachineProcedure.Flow
+      executeFromState(BasicProcedureEnv env, ServerCrashState state)
+        throws ProcedureYieldException, InterruptedException {
       return null;
     }
 
     @Override
-    protected void rollbackState(BasicProcedureEnv env, ServerCrashState state) throws IOException,
-        InterruptedException {
+    protected void rollbackState(BasicProcedureEnv env, ServerCrashState state)
+      throws IOException, InterruptedException {
     }
 
     @Override
