@@ -33,6 +33,8 @@ import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.NamespaceDescriptor;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableName;
+import org.apache.hadoop.hbase.client.BalanceRequest;
+import org.apache.hadoop.hbase.client.BalanceResponse;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.TableDescriptor;
 import org.apache.hadoop.hbase.client.TableDescriptorBuilder;
@@ -509,32 +511,37 @@ public class RSGroupAdminServer implements RSGroupAdmin {
   }
 
   @Override
-  public boolean balanceRSGroup(String groupName) throws IOException {
+  public BalanceResponse balanceRSGroup(String groupName, BalanceRequest request)
+    throws IOException {
     ServerManager serverManager = master.getServerManager();
     LoadBalancer balancer = master.getLoadBalancer();
 
+    BalanceResponse.Builder responseBuilder = BalanceResponse.newBuilder();
+
     synchronized (balancer) {
       // If balance not true, don't run balancer.
-      if (!((HMaster) master).isBalancerOn()) {
-        return false;
+      if (!((HMaster) master).isBalancerOn() && !request.isDryRun()) {
+        return responseBuilder.build();
       }
 
       if (getRSGroupInfo(groupName) == null) {
         throw new ConstraintException("RSGroup does not exist: " + groupName);
       }
+
       // Only allow one balance run at at time.
       Map<String, RegionState> groupRIT = rsGroupGetRegionsInTransition(groupName);
-      if (groupRIT.size() > 0) {
+      if (groupRIT.size() > 0 && !request.isIgnoreRegionsInTransition()) {
         LOG.debug("Not running balancer because {} region(s) in transition: {}", groupRIT.size(),
           StringUtils.abbreviate(
             master.getAssignmentManager().getRegionStates().getRegionsInTransition().toString(),
             256));
-        return false;
+        return responseBuilder.build();
       }
+
       if (serverManager.areDeadServersInProgress()) {
         LOG.debug("Not running balancer because processing dead regionserver(s): {}",
           serverManager.getDeadServers());
-        return false;
+        return responseBuilder.build();
       }
 
       // We balance per group instead of per table
@@ -542,12 +549,17 @@ public class RSGroupAdminServer implements RSGroupAdmin {
         getRSGroupAssignmentsByTable(master.getTableStateManager(), groupName);
       List<RegionPlan> plans = balancer.balanceCluster(assignmentsByTable);
       boolean balancerRan = !plans.isEmpty();
-      if (balancerRan) {
+
+      responseBuilder.setBalancerRan(balancerRan).setMovesCalculated(plans.size());
+
+      if (balancerRan && !request.isDryRun()) {
         LOG.info("RSGroup balance {} starting with plan count: {}", groupName, plans.size());
-        master.executeRegionPlansWithThrottling(plans);
+        List<RegionPlan> executed = master.executeRegionPlansWithThrottling(plans);
+        responseBuilder.setMovesExecuted(executed.size());
         LOG.info("RSGroup balance " + groupName + " completed");
       }
-      return balancerRan;
+
+      return responseBuilder.build();
     }
   }
 
@@ -631,6 +643,15 @@ public class RSGroupAdminServer implements RSGroupAdmin {
     }
   }
 
+  /**
+   * Because the {@link RSGroupAdminClient#updateConfiguration(String)} calls
+   * {@link org.apache.hadoop.hbase.client.Admin#updateConfiguration(ServerName)} method, the
+   * implementation of this method on the Server side is empty.
+   */
+  @Override
+  public void updateConfiguration(String groupName) throws IOException {
+  }
+
   private Map<String, RegionState> rsGroupGetRegionsInTransition(String groupName)
     throws IOException {
     Map<String, RegionState> rit = Maps.newTreeMap();
@@ -673,9 +694,8 @@ public class RSGroupAdminServer implements RSGroupAdmin {
         if (currRegion.isSplitParent()) {
           continue;
         }
-        assignments.putIfAbsent(currTable, new HashMap<>());
-        assignments.get(currTable).putIfAbsent(currServer, new ArrayList<>());
-        assignments.get(currTable).get(currServer).add(currRegion);
+        assignments.computeIfAbsent(currTable, key -> new HashMap<>())
+          .computeIfAbsent(currServer, key -> new ArrayList<>()).add(currRegion);
       }
     }
 
@@ -689,10 +709,14 @@ public class RSGroupAdminServer implements RSGroupAdmin {
     // add all tables that are members of the group
     for (TableName tableName : rsGroupInfo.getTables()) {
       if (assignments.containsKey(tableName)) {
-        result.put(tableName, new HashMap<>());
-        result.get(tableName).putAll(serverMap);
-        result.get(tableName).putAll(assignments.get(tableName));
-        LOG.debug("Adding assignments for {}: {}", tableName, assignments.get(tableName));
+        Map<ServerName, List<RegionInfo>> tableResults = new HashMap<>(serverMap);
+
+        Map<ServerName, List<RegionInfo>> tableAssignments = assignments.get(tableName);
+        tableResults.putAll(tableAssignments);
+
+        result.put(tableName, tableResults);
+
+        LOG.debug("Adding assignments for {}: {}", tableName, tableAssignments);
       }
     }
 
