@@ -22,6 +22,7 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
 import java.lang.reflect.Field;
+import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
@@ -45,6 +46,7 @@ import org.apache.hadoop.hbase.net.Address;
 import org.apache.hadoop.hbase.testclassification.LargeTests;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.JVMClusterUtil;
+import org.apache.hadoop.hbase.util.ReflectionUtils;
 import org.apache.hadoop.hbase.util.VersionInfo;
 import org.junit.After;
 import org.junit.AfterClass;
@@ -264,27 +266,24 @@ public class TestRSGroupsKillRS extends TestRSGroupsBase {
     Address address = servers.iterator().next();
     int majorVersion = VersionInfo.getMajorVersion(originVersion);
     assertTrue(majorVersion >= 1);
-    String lowerVersion =
-      String.valueOf(majorVersion - 1) + originVersion.substring(originVersion.indexOf("."));
-    try {
-      setVersionInfoVersion(lowerVersion);
-      TEST_UTIL.getMiniHBaseCluster().startRegionServer(address.getHostName(), address.getPort());
-      assertEquals(NUM_SLAVES_BASE,
-        TEST_UTIL.getMiniHBaseCluster().getLiveRegionServerThreads().size());
-      assertTrue(VersionInfo.compareVersion(originVersion,
-        master.getRegionServerVersion(getServerName(servers.iterator().next()))) > 0);
-      LOG.debug("wait for META assigned...");
-      // SCP finished, which means all regions assigned too.
-      TEST_UTIL.waitFor(60000, () -> !TEST_UTIL.getHBaseCluster().getMaster().getProcedures()
-        .stream().filter(p -> (p instanceof ServerCrashProcedure)).findAny().isPresent());
-    } finally {
-      setVersionInfoVersion(Version.version);
-    }
+    String lowerVersion = String.valueOf(majorVersion - 1) + originVersion.split("\\.")[1];
+    setFinalStatic(Version.class.getField("version"), lowerVersion);
+    TEST_UTIL.getMiniHBaseCluster().startRegionServer(address.getHostName(), address.getPort());
+    assertEquals(NUM_SLAVES_BASE,
+      TEST_UTIL.getMiniHBaseCluster().getLiveRegionServerThreads().size());
+    assertTrue(VersionInfo.compareVersion(originVersion,
+      master.getRegionServerVersion(getServerName(servers.iterator().next()))) > 0);
+    LOG.debug("wait for META assigned...");
+    // SCP finished, which means all regions assigned too.
+    TEST_UTIL.waitFor(60000, () -> !TEST_UTIL.getHBaseCluster().getMaster().getProcedures().stream()
+      .filter(p -> (p instanceof ServerCrashProcedure)).findAny().isPresent());
   }
 
-  private static void setVersionInfoVersion(String newValue) throws Exception {
-    Field f = VersionInfo.class.getDeclaredField("version");
-    f.setAccessible(true);
-    f.set(null, newValue);
+  private static void setFinalStatic(Field field, Object newValue) throws Exception {
+    field.setAccessible(true);
+    Field modifiersField = ReflectionUtils.getModifiersField();
+    modifiersField.setAccessible(true);
+    modifiersField.setInt(field, field.getModifiers() & ~Modifier.FINAL);
+    field.set(null, newValue);
   }
 }
