@@ -18,6 +18,8 @@
 package org.apache.hadoop.hbase.ipc;
 
 import java.io.IOException;
+import java.util.Map;
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.CellScanner;
 import org.apache.hadoop.hbase.HBaseInterfaceAudience;
 import org.apache.yetus.audience.InterfaceAudience;
@@ -82,6 +84,29 @@ public interface RpcCall extends RpcCallContext {
   /** Returns The request header of this call. */
   RequestHeader getHeader();
 
+  /**
+   * Returns the map of attributes specified when building the Connection.
+   * @see org.apache.hadoop.hbase.client.ConnectionFactory#createConnection(Configuration,
+   *      ExecutorService, User, Map)
+   */
+  Map<String, byte[]> getConnectionAttributes();
+
+  /**
+   * Returns the map of attributes specified when building the request. This map is lazily evaluated
+   * so if you only need a single attribute then it may be cheaper to use
+   * {@link #getRequestAttribute(String)}
+   * @see org.apache.hadoop.hbase.client.TableBuilder#setRequestAttribute(String, byte[])
+   */
+  Map<String, byte[]> getRequestAttributes();
+
+  /**
+   * Returns a single request attribute value, or null if no value is present. If you need many
+   * request attributes then you should fetch the lazily evaluated map via
+   * {@link #getRequestAttributes()}
+   * @see org.apache.hadoop.hbase.client.TableBuilder#setRequestAttribute(String, byte[])
+   */
+  byte[] getRequestAttribute(String key);
+
   /** Returns Port of remote address in this call */
   int getRemotePort();
 
@@ -96,7 +121,7 @@ public interface RpcCall extends RpcCallContext {
 
   /**
    * Send the response of this RPC call. Implementation provides the underlying facility
-   * (connection, etc) to send. n
+   * (connection, etc) to send.
    */
   void sendResponseIfReady() throws IOException;
 
@@ -107,4 +132,8 @@ public interface RpcCall extends RpcCallContext {
 
   /** Returns A short string format of this call without possibly lengthy params */
   String toShortString();
+
+  void updateFsReadTime(long latencyMillis);
+
+  long getFsReadTime();
 }
