@@ -722,12 +722,27 @@ public class SnapshotManager extends MasterProcedureManager implements Stoppable
       .submitProcedure(new MasterProcedureUtil.NonceProcedureRunnable(master, nonceGroup, nonce) {
         @Override
         protected void run() throws IOException {
+          TableDescriptor tableDescriptor =
+            master.getTableDescriptors().get(TableName.valueOf(snapshot.getTable()));
+          MasterCoprocessorHost cpHost = getMaster().getMasterCoprocessorHost();
+          User user = RpcServer.getRequestUser().orElse(null);
+          org.apache.hadoop.hbase.client.SnapshotDescription snapshotDesc =
+            ProtobufUtil.createSnapshotDesc(snapshot);
+
+          if (cpHost != null) {
+            cpHost.preSnapshot(snapshotDesc, tableDescriptor, user);
+          }
+
           sanityCheckBeforeSnapshot(snapshot, false);
 
           long procId = submitProcedure(new SnapshotProcedure(
             getMaster().getMasterProcedureExecutor().getEnvironment(), snapshot));
 
           getMaster().getSnapshotManager().registerSnapshotProcedure(snapshot, procId);
+
+          if (cpHost != null) {
+            cpHost.postSnapshot(snapshotDesc, tableDescriptor, user);
+          }
         }
 
         @Override
