@@ -20,10 +20,13 @@ package org.apache.hadoop.hbase.quotas;
 import java.util.Arrays;
 import java.util.List;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.hbase.Cell;
+import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.client.Mutation;
 import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.ipc.RpcCall;
 import org.apache.hadoop.hbase.ipc.RpcServer;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.yetus.audience.InterfaceStability;
 
@@ -44,11 +47,18 @@ public class DefaultOperationQuota implements OperationQuota {
 
   // the available read/write quota size in bytes
   protected long readAvailable = 0;
+
+  // The estimated handler usage time in ms for a request based on
+  // the number of requests per second and the number of handler threads
+  private final long estimatedHandlerUsagePerReq;
+
   // estimated quota
   protected long writeConsumed = 0;
   protected long readConsumed = 0;
   protected long writeCapacityUnitConsumed = 0;
   protected long readCapacityUnitConsumed = 0;
+  protected long handlerUsageTimeConsumed = 0;
+
   // real consumed quota
   private final long[] operationSize;
   // difference between estimated quota and real consumed quota used in close method
@@ -58,13 +68,15 @@ public class DefaultOperationQuota implements OperationQuota {
   protected long readDiff = 0;
   protected long writeCapacityUnitDiff = 0;
   protected long readCapacityUnitDiff = 0;
+  protected long handlerUsageTimeDiff = 0;
   private boolean useResultSizeBytes;
   private long blockSizeBytes;
   private long maxScanEstimate;
+  private boolean isAtomic = false;
 
   public DefaultOperationQuota(final Configuration conf, final int blockSizeBytes,
-    final QuotaLimiter... limiters) {
-    this(conf, Arrays.asList(limiters));
+    final double requestsPerSecond, final QuotaLimiter... limiters) {
+    this(conf, requestsPerSecond, Arrays.asList(limiters));
     this.useResultSizeBytes =
       conf.getBoolean(OperationQuota.USE_RESULT_SIZE_BYTES, USE_RESULT_SIZE_BYTES_DEFAULT);
     this.blockSizeBytes = blockSizeBytes;
@@ -76,24 +88,30 @@ public class DefaultOperationQuota implements OperationQuota {
   /**
    * NOTE: The order matters. It should be something like [user, table, namespace, global]
    */
-  public DefaultOperationQuota(final Configuration conf, final List<QuotaLimiter> limiters) {
+  public DefaultOperationQuota(final Configuration conf, final double requestsPerSecond,
+    final List<QuotaLimiter> limiters) {
     this.writeCapacityUnit =
       conf.getLong(QuotaUtil.WRITE_CAPACITY_UNIT_CONF_KEY, QuotaUtil.DEFAULT_WRITE_CAPACITY_UNIT);
     this.readCapacityUnit =
       conf.getLong(QuotaUtil.READ_CAPACITY_UNIT_CONF_KEY, QuotaUtil.DEFAULT_READ_CAPACITY_UNIT);
     this.limiters = limiters;
+    int numHandlerThreads = conf.getInt(HConstants.REGION_SERVER_HANDLER_COUNT,
+      HConstants.DEFAULT_REGION_SERVER_HANDLER_COUNT);
+    this.estimatedHandlerUsagePerReq =
+      calculateHandlerUsageTimeEstimate(requestsPerSecond, numHandlerThreads);
+
     int size = OperationType.values().length;
     operationSize = new long[size];
-
     for (int i = 0; i < size; ++i) {
       operationSize[i] = 0;
     }
   }
 
   @Override
-  public void checkBatchQuota(int numWrites, int numReads) throws RpcThrottlingException {
+  public void checkBatchQuota(int numWrites, int numReads, boolean isAtomic)
+    throws RpcThrottlingException {
     updateEstimateConsumeBatchQuota(numWrites, numReads);
-    checkQuota(numWrites, numReads);
+    checkQuota(numWrites, numReads, isAtomic);
   }
 
   @Override
@@ -101,24 +119,37 @@ public class DefaultOperationQuota implements OperationQuota {
     long maxBlockBytesScanned, long prevBlockBytesScannedDifference) throws RpcThrottlingException {
     updateEstimateConsumeScanQuota(scanRequest, maxScannerResultSize, maxBlockBytesScanned,
       prevBlockBytesScannedDifference);
-    checkQuota(0, 1);
+    checkQuota(0, 1, false);
   }
 
-  private void checkQuota(long numWrites, long numReads) throws RpcThrottlingException {
+  private void checkQuota(long numWrites, long numReads, boolean isAtomic)
+    throws RpcThrottlingException {
+    if (isAtomic) {
+      // Remember this flag for later use in close()
+      this.isAtomic = true;
+    }
     readAvailable = Long.MAX_VALUE;
     for (final QuotaLimiter limiter : limiters) {
       if (limiter.isBypass()) {
         continue;
       }
 
-      limiter.checkQuota(numWrites, writeConsumed, numReads, readConsumed,
-        writeCapacityUnitConsumed, readCapacityUnitConsumed);
+      long maxRequestsToEstimate = limiter.getRequestNumLimit();
+      long maxReadsToEstimate = Math.min(maxRequestsToEstimate, limiter.getReadNumLimit());
+      long maxWritesToEstimate = Math.min(maxRequestsToEstimate, limiter.getWriteNumLimit());
+      long maxReadSizeToEstimate = Math.min(readConsumed, limiter.getReadLimit());
+      long maxWriteSizeToEstimate = Math.min(writeConsumed, limiter.getWriteLimit());
+
+      limiter.checkQuota(Math.min(maxWritesToEstimate, numWrites),
+        Math.min(maxWriteSizeToEstimate, writeConsumed), Math.min(maxReadsToEstimate, numReads),
+        Math.min(maxReadSizeToEstimate, readConsumed), writeCapacityUnitConsumed,
+        readCapacityUnitConsumed, isAtomic, handlerUsageTimeConsumed);
       readAvailable = Math.min(readAvailable, limiter.getReadAvailable());
     }
 
     for (final QuotaLimiter limiter : limiters) {
       limiter.grabQuota(numWrites, writeConsumed, numReads, readConsumed, writeCapacityUnitConsumed,
-        readCapacityUnitConsumed);
+        readCapacityUnitConsumed, isAtomic, handlerUsageTimeConsumed);
     }
   }
 
@@ -136,19 +167,22 @@ public class DefaultOperationQuota implements OperationQuota {
         RpcServer.getCurrentCall().map(RpcCall::getBlockBytesScanned).orElse(0L);
       readDiff = Math.max(blockBytesScanned, resultSize) - readConsumed;
     }
-
     writeCapacityUnitDiff =
       calculateWriteCapacityUnitDiff(operationSize[OperationType.MUTATE.ordinal()], writeConsumed);
     readCapacityUnitDiff = calculateReadCapacityUnitDiff(
       operationSize[OperationType.GET.ordinal()] + operationSize[OperationType.SCAN.ordinal()],
       readConsumed);
+    handlerUsageTimeDiff = calculateHandlerUsageMsDiff();
 
     for (final QuotaLimiter limiter : limiters) {
       if (writeDiff != 0) {
-        limiter.consumeWrite(writeDiff, writeCapacityUnitDiff);
+        limiter.consumeWrite(writeDiff, writeCapacityUnitDiff, isAtomic);
       }
       if (readDiff != 0) {
-        limiter.consumeRead(readDiff, readCapacityUnitDiff);
+        limiter.consumeRead(readDiff, readCapacityUnitDiff, isAtomic);
+      }
+      if (handlerUsageTimeDiff != 0) {
+        limiter.consumeTime(handlerUsageTimeDiff);
       }
     }
   }
@@ -173,6 +207,11 @@ public class DefaultOperationQuota implements OperationQuota {
     operationSize[OperationType.SCAN.ordinal()] += QuotaUtil.calculateResultSize(results);
   }
 
+  @Override
+  public void addScanResultCells(final List<Cell> cells) {
+    operationSize[OperationType.SCAN.ordinal()] += QuotaUtil.calculateCellsSize(cells);
+  }
+
   @Override
   public void addMutation(final Mutation mutation) {
     operationSize[OperationType.MUTATE.ordinal()] += QuotaUtil.calculateMutationSize(mutation);
@@ -195,6 +234,8 @@ public class DefaultOperationQuota implements OperationQuota {
 
     writeCapacityUnitConsumed = calculateWriteCapacityUnit(writeConsumed);
     readCapacityUnitConsumed = calculateReadCapacityUnit(readConsumed);
+
+    handlerUsageTimeConsumed = (numReads + numWrites) * estimatedHandlerUsagePerReq;
   }
 
   /**
@@ -217,6 +258,7 @@ public class DefaultOperationQuota implements OperationQuota {
     }
 
     readCapacityUnitConsumed = calculateReadCapacityUnit(readConsumed);
+    handlerUsageTimeConsumed = estimatedHandlerUsagePerReq;
   }
 
   protected static long getScanReadConsumeEstimate(long blockSizeBytes, long nextCallSeq,
@@ -267,4 +309,25 @@ public class DefaultOperationQuota implements OperationQuota {
   private long calculateReadCapacityUnitDiff(final long actualSize, final long estimateSize) {
     return calculateReadCapacityUnit(actualSize) - calculateReadCapacityUnit(estimateSize);
   }
+
+  private long calculateHandlerUsageTimeEstimate(final double requestsPerSecond,
+    final int numHandlerThreads) {
+    if (requestsPerSecond <= numHandlerThreads) {
+      // If less than 1 request per second per handler thread, then we use the number of handler
+      // threads as a baseline to avoid incorrect estimations when the number of requests is very
+      // low.
+      return numHandlerThreads;
+    } else {
+      double requestsPerMillisecond = Math.ceil(requestsPerSecond / 1000);
+      // We don't ever want zero here
+      return Math.max((long) requestsPerMillisecond, 1L);
+    }
+  }
+
+  private long calculateHandlerUsageMsDiff() {
+    long currentTime = EnvironmentEdgeManager.currentTime();
+    long startTime = RpcServer.getCurrentCall().map(RpcCall::getStartTime).orElse(currentTime);
+    long timeElapsed = currentTime - startTime;
+    return handlerUsageTimeConsumed - timeElapsed;
+  }
 }
