@@ -95,6 +95,14 @@ public class QuotaUtil extends QuotaTableUtil {
     "hbase.quota.default.user.machine.write.num";
   public static final String QUOTA_DEFAULT_USER_MACHINE_WRITE_SIZE =
     "hbase.quota.default.user.machine.write.size";
+  public static final String QUOTA_DEFAULT_USER_MACHINE_ATOMIC_READ_SIZE =
+    "hbase.quota.default.user.machine.atomic.read.size";
+  public static final String QUOTA_DEFAULT_USER_MACHINE_ATOMIC_REQUEST_NUM =
+    "hbase.quota.default.user.machine.atomic.request.num";
+  public static final String QUOTA_DEFAULT_USER_MACHINE_ATOMIC_WRITE_SIZE =
+    "hbase.quota.default.user.machine.atomic.write.size";
+  public static final String QUOTA_DEFAULT_USER_MACHINE_REQUEST_HANDLER_USAGE_MS =
+    "hbase.quota.default.user.machine.request.handler.usage.ms";
 
   /** Table descriptor for Quota internal table */
   public static final HTableDescriptor QUOTA_TABLE_DESC = new HTableDescriptor(QUOTA_TABLE_NAME);
@@ -388,6 +396,14 @@ public class QuotaUtil extends QuotaTableUtil {
       .ifPresent(throttleBuilder::setWriteNum);
     buildDefaultTimedQuota(conf, QUOTA_DEFAULT_USER_MACHINE_WRITE_SIZE)
       .ifPresent(throttleBuilder::setWriteSize);
+    buildDefaultTimedQuota(conf, QUOTA_DEFAULT_USER_MACHINE_ATOMIC_READ_SIZE)
+      .ifPresent(throttleBuilder::setAtomicReadSize);
+    buildDefaultTimedQuota(conf, QUOTA_DEFAULT_USER_MACHINE_ATOMIC_REQUEST_NUM)
+      .ifPresent(throttleBuilder::setAtomicReqNum);
+    buildDefaultTimedQuota(conf, QUOTA_DEFAULT_USER_MACHINE_ATOMIC_WRITE_SIZE)
+      .ifPresent(throttleBuilder::setAtomicWriteSize);
+    buildDefaultTimedQuota(conf, QUOTA_DEFAULT_USER_MACHINE_REQUEST_HANDLER_USAGE_MS)
+      .ifPresent(throttleBuilder::setReqHandlerUsageMs);
 
     UserQuotaState state = new UserQuotaState(nowTs);
     QuotaProtos.Quotas defaultQuotas =
@@ -590,6 +606,14 @@ public class QuotaUtil extends QuotaTableUtil {
     return size;
   }
 
+  public static long calculateCellsSize(final List<Cell> cells) {
+    long size = 0;
+    for (Cell cell : cells) {
+      size += cell.getSerializedSize();
+    }
+    return size;
+  }
+
   /**
    * Method to enable a table, if not already enabled. This method suppresses
    * {@link TableNotDisabledException} and {@link TableNotFoundException}, if thrown while enabling
