@@ -28,7 +28,6 @@ import java.io.IOException;
 import java.util.Map;
 import java.util.Optional;
 import java.util.SortedSet;
-import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.CellComparator;
@@ -38,6 +37,7 @@ import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.PrivateCellUtil;
 import org.apache.hadoop.hbase.client.Scan;
 import org.apache.hadoop.hbase.io.TimeRange;
+import org.apache.hadoop.hbase.io.encoding.DataBlockEncoding;
 import org.apache.hadoop.hbase.io.hfile.BlockType;
 import org.apache.hadoop.hbase.io.hfile.BloomFilterMetrics;
 import org.apache.hadoop.hbase.io.hfile.CacheConfig;
@@ -68,7 +68,7 @@ public class StoreFileReader {
   protected BloomFilter deleteFamilyBloomFilter = null;
   private BloomFilterMetrics bloomFilterMetrics = null;
   protected BloomType bloomFilterType;
-  private final HFile.Reader reader;
+  protected final HFile.Reader reader;
   protected long sequenceID = -1;
   protected TimeRange timeRange = null;
   private byte[] lastBloomKey;
@@ -79,24 +79,26 @@ public class StoreFileReader {
   private int prefixLength = -1;
   protected Configuration conf;
 
-  // Counter that is incremented every time a scanner is created on the
-  // store file. It is decremented when the scan on the store file is
-  // done. All StoreFileReader for the same StoreFile will share this counter.
-  private final AtomicInteger refCount;
+  /**
+   * All {@link StoreFileReader} for the same StoreFile will share the
+   * {@link StoreFileInfo#refCount}. Counter that is incremented every time a scanner is created on
+   * the store file. It is decremented when the scan on the store file is done.
+   */
+  private final StoreFileInfo storeFileInfo;
   private final ReaderContext context;
 
-  private StoreFileReader(HFile.Reader reader, AtomicInteger refCount, ReaderContext context,
+  private StoreFileReader(HFile.Reader reader, StoreFileInfo storeFileInfo, ReaderContext context,
     Configuration conf) {
     this.reader = reader;
     bloomFilterType = BloomType.NONE;
-    this.refCount = refCount;
+    this.storeFileInfo = storeFileInfo;
     this.context = context;
     this.conf = conf;
   }
 
   public StoreFileReader(ReaderContext context, HFileInfo fileInfo, CacheConfig cacheConf,
-    AtomicInteger refCount, Configuration conf) throws IOException {
-    this(HFile.createReader(context, fileInfo, cacheConf, conf), refCount, context, conf);
+    StoreFileInfo storeFileInfo, Configuration conf) throws IOException {
+    this(HFile.createReader(context, fileInfo, cacheConf, conf), storeFileInfo, context, conf);
   }
 
   void copyFields(StoreFileReader storeFileReader) throws IOException {
@@ -122,7 +124,7 @@ public class StoreFileReader {
    */
   @InterfaceAudience.Private
   StoreFileReader() {
-    this.refCount = new AtomicInteger(0);
+    this.storeFileInfo = null;
     this.reader = null;
     this.context = null;
   }
@@ -145,7 +147,8 @@ public class StoreFileReader {
   public StoreFileScanner getStoreFileScanner(boolean cacheBlocks, boolean pread,
     boolean isCompaction, long readPt, long scannerOrder, boolean canOptimizeForNonNullColumn) {
     return new StoreFileScanner(this, getScanner(cacheBlocks, pread, isCompaction), !isCompaction,
-      reader.hasMVCCInfo(), readPt, scannerOrder, canOptimizeForNonNullColumn);
+      reader.hasMVCCInfo(), readPt, scannerOrder, canOptimizeForNonNullColumn,
+      reader.getDataBlockEncoding() == DataBlockEncoding.ROW_INDEX_V1);
   }
 
   /**
@@ -153,7 +156,7 @@ public class StoreFileReader {
    * is opened.
    */
   int getRefCount() {
-    return refCount.get();
+    return storeFileInfo.getRefCount();
   }
 
   /**
@@ -161,7 +164,7 @@ public class StoreFileReader {
    * count so reader is not close until some object is holding the lock
    */
   void incrementRefCount() {
-    refCount.incrementAndGet();
+    storeFileInfo.increaseRefCount();
   }
 
   /**
@@ -169,7 +172,7 @@ public class StoreFileReader {
    * count, and also, if this is not the common pread reader, we should close it.
    */
   void readCompleted() {
-    refCount.decrementAndGet();
+    storeFileInfo.decreaseRefCount();
     if (context.getReaderType() == ReaderType.STREAM) {
       try {
         reader.close(false);
@@ -402,7 +405,7 @@ public class StoreFileReader {
         // of the hbase:meta cells. We can safely use Bytes.BYTES_RAWCOMPARATOR for ROW Bloom
         if (keyIsAfterLast) {
           if (bloomFilterType == BloomType.ROWCOL) {
-            keyIsAfterLast = (CellComparator.getInstance().compare(kvKey, lastBloomKeyOnlyKV)) > 0;
+            keyIsAfterLast = (getComparator().compare(kvKey, lastBloomKeyOnlyKV)) > 0;
           } else {
             keyIsAfterLast = (Bytes.BYTES_RAWCOMPARATOR.compare(key, lastBloomKey) > 0);
           }
@@ -416,10 +419,7 @@ public class StoreFileReader {
           Cell rowBloomKey = PrivateCellUtil.createFirstOnRow(kvKey);
           // hbase:meta does not have blooms. So we need not have special interpretation
           // of the hbase:meta cells. We can safely use Bytes.BYTES_RAWCOMPARATOR for ROW Bloom
-          if (
-            keyIsAfterLast
-              && (CellComparator.getInstance().compare(rowBloomKey, lastBloomKeyOnlyKV)) > 0
-          ) {
+          if (keyIsAfterLast && (getComparator().compare(rowBloomKey, lastBloomKeyOnlyKV)) > 0) {
             exists = false;
           } else {
             exists = bloomFilter.contains(kvKey, bloom, BloomType.ROWCOL)
@@ -635,8 +635,7 @@ public class StoreFileReader {
     return this.bulkLoadResult;
   }
 
-  @InterfaceAudience.LimitedPrivate(HBaseInterfaceAudience.UNITTEST)
-  public BloomFilter getGeneralBloomFilter() {
+  BloomFilter getGeneralBloomFilter() {
     return generalBloomFilter;
   }
 
