@@ -390,6 +390,9 @@ public class FSHLog extends AbstractFSWAL<Writer> {
             closeWriter(this.writer, oldPath, true);
           } finally {
             inflightWALClosures.remove(oldPath.getName());
+            if (!isUnflushedEntries()) {
+              markClosedAndClean(oldPath);
+            }
           }
         } else {
           Writer localWriter = this.writer;
@@ -1208,4 +1211,16 @@ public class FSHLog extends AbstractFSWAL<Writer> {
     }
     return new DatanodeInfo[0];
   }
+
+  // Visible for testing
+  @InterfaceAudience.Private
+  void setWriter(Writer writer) {
+    this.writer = writer;
+  }
+
+  // Visible for testing
+  @InterfaceAudience.Private
+  protected int getClosedErrorCount() {
+    return closeErrorCount.get();
+  }
 }
