@@ -17,10 +17,13 @@
  */
 package org.apache.hadoop.hbase.regionserver.wal;
 
+import static org.apache.hadoop.hbase.HConstants.REPLICATION_SCOPE_GLOBAL;
+
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Map;
 import java.util.NavigableMap;
+import java.util.TreeMap;
 import java.util.function.Function;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
@@ -28,6 +31,7 @@ import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.regionserver.MultiVersionConcurrencyControl;
+import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.CommonFSUtils;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.wal.WAL;
@@ -169,6 +173,12 @@ public class WALUtil {
       if (walKey.getWriteEntry() != null) {
         mvcc.complete(walKey.getWriteEntry());
       }
+      /**
+       * Here we do not abort the RegionServer for {@link WALSyncTimeoutIOException} as
+       * {@link HRegion#doWALAppend} does,because WAL Marker just records the internal state and
+       * seems it is no need to always abort the RegionServer when {@link WAL#sync} timeout,it is
+       * the internal state transition that determines whether RegionServer is aborted or not.
+       */
       throw ioe;
     }
     return walKey;
@@ -217,4 +227,12 @@ public class WALUtil {
       cells.trimToSize();
     }
   }
+
+  public static void writeReplicationMarkerAndSync(WAL wal, MultiVersionConcurrencyControl mvcc,
+    RegionInfo regionInfo, byte[] rowKey, long timestamp) throws IOException {
+    NavigableMap<byte[], Integer> replicationScope = new TreeMap<>(Bytes.BYTES_COMPARATOR);
+    replicationScope.put(WALEdit.METAFAMILY, REPLICATION_SCOPE_GLOBAL);
+    writeMarker(wal, replicationScope, regionInfo,
+      WALEdit.createReplicationMarkerEdit(rowKey, timestamp), mvcc, null);
+  }
 }
