@@ -428,9 +428,7 @@ public class LoadIncrementalHFiles extends Configured implements Tool {
       LOG.warn("Secure bulk load has been integrated into HBase core.");
     }
 
-    Path path = queue.peek().getFilePath();
-    FileSystem fs = path.getFileSystem(getConf()).resolvePath(path).getFileSystem(getConf());
-    fsDelegationToken.acquireDelegationToken(fs);
+    fsDelegationToken.acquireDelegationToken(queue.peek().getFilePath().getFileSystem(getConf()));
     bulkToken = secureClient.prepareBulkLoad(admin.getConnection());
     Pair<Multimap<ByteBuffer, LoadQueueItem>, Set<String>> pair = null;
 
@@ -551,7 +549,7 @@ public class LoadIncrementalHFiles extends Configured implements Tool {
       lqis.stream().map(lqi -> Pair.newPair(lqi.getFamily(), lqi.getFilePath().toString()))
         .collect(Collectors.toList());
     return new ClientServiceCallable<byte[]>(conn, tableName, first,
-      rpcControllerFactory.newController(), HConstants.PRIORITY_UNSET) {
+      rpcControllerFactory.newController(), HConstants.PRIORITY_UNSET, Collections.emptyMap()) {
       @Override
       protected byte[] rpcCall() throws Exception {
         SecureBulkLoadClient secureClient = null;
@@ -1171,10 +1169,11 @@ public class LoadIncrementalHFiles extends Configured implements Tool {
     StoreFileWriter halfWriter = null;
     try {
       ReaderContext context = new ReaderContextBuilder().withFileSystemAndPath(fs, inFile).build();
-      HFileInfo hfile = new HFileInfo(context, conf);
-      halfReader =
-        new HalfStoreFileReader(context, hfile, cacheConf, reference, new AtomicInteger(0), conf);
-      hfile.initMetaAndIndex(halfReader.getHFileReader());
+      StoreFileInfo storeFileInfo =
+        new StoreFileInfo(conf, fs, fs.getFileStatus(inFile), reference);
+      storeFileInfo.initHFileInfo(context);
+      halfReader = (HalfStoreFileReader) storeFileInfo.createReader(context, cacheConf);
+      storeFileInfo.getHFileInfo().initMetaAndIndex(halfReader.getHFileReader());
       Map<byte[], byte[]> fileInfo = halfReader.loadFileInfo();
 
       int blocksize = familyDescriptor.getBlocksize();
