@@ -28,16 +28,18 @@ import org.apache.hadoop.hbase.Abortable;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.io.asyncfs.monitor.ExcludeDatanodeManager;
 import org.apache.hadoop.hbase.regionserver.wal.MetricsWAL;
-import org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader;
+import org.apache.hadoop.hbase.regionserver.wal.ProtobufWALStreamReader;
+import org.apache.hadoop.hbase.regionserver.wal.ProtobufWALTailingReader;
 import org.apache.hadoop.hbase.util.CancelableProgressable;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.LeaseNotRecoveredException;
-import org.apache.hadoop.hbase.wal.WAL.Reader;
 import org.apache.hadoop.hbase.wal.WALProvider.Writer;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hbase.thirdparty.com.google.common.base.Preconditions;
+
 /**
  * Entry point for users of the Write Ahead Log. Acts as the shim between internal use and the
  * particular WALProvider we use to handle wal requests. Configure which provider gets used with the
@@ -57,6 +59,21 @@ import org.slf4j.LoggerFactory;
 @InterfaceAudience.Private
 public class WALFactory {
 
+  /**
+   * Used in tests for injecting customized stream reader implementation, for example, inject fault
+   * when reading, etc.
+   * <p/>
+   * After removing the sequence file based WAL, we always use protobuf based WAL reader, and we
+   * will also determine whether the WAL file is encrypted and we should use
+   * {@link org.apache.hadoop.hbase.regionserver.wal.SecureWALCellCodec} to decode by check the
+   * header of the WAL file, so we do not need to specify a specical reader to read the WAL file
+   * either.
+   * <p/>
+   * So typically you should not use this config in production.
+   */
+  public static final String WAL_STREAM_READER_CLASS_IMPL =
+    "hbase.regionserver.wal.stream.reader.impl";
+
   private static final Logger LOG = LoggerFactory.getLogger(WALFactory.class);
 
   /**
@@ -93,7 +110,7 @@ public class WALFactory {
   /**
    * Configuration-specified WAL Reader used when a custom reader is requested
    */
-  private final Class<? extends AbstractFSWALProvider.Reader> logReaderClass;
+  private final Class<? extends WALStreamReader> walStreamReaderClass;
 
   /**
    * How long to attempt opening in-recovery wals
@@ -111,8 +128,12 @@ public class WALFactory {
     // happen prior to provider initialization, in case they need to instantiate a reader/writer.
     timeoutMillis = conf.getInt("hbase.hlog.open.timeout", 300000);
     /* TODO Both of these are probably specific to the fs wal provider */
-    logReaderClass = conf.getClass("hbase.regionserver.hlog.reader.impl", ProtobufLogReader.class,
-      AbstractFSWALProvider.Reader.class);
+    walStreamReaderClass = conf.getClass(WAL_STREAM_READER_CLASS_IMPL,
+      ProtobufWALStreamReader.class, WALStreamReader.class);
+    Preconditions.checkArgument(
+      AbstractFSWALProvider.Initializer.class.isAssignableFrom(walStreamReaderClass),
+      "The wal stream reader class %s is not a sub class of %s", walStreamReaderClass.getName(),
+      AbstractFSWALProvider.Initializer.class.getName());
     this.conf = conf;
     // end required early initialization
 
@@ -196,8 +217,12 @@ public class WALFactory {
     // happen prior to provider initialization, in case they need to instantiate a reader/writer.
     timeoutMillis = conf.getInt("hbase.hlog.open.timeout", 300000);
     /* TODO Both of these are probably specific to the fs wal provider */
-    logReaderClass = conf.getClass("hbase.regionserver.hlog.reader.impl", ProtobufLogReader.class,
-      AbstractFSWALProvider.Reader.class);
+    walStreamReaderClass = conf.getClass(WAL_STREAM_READER_CLASS_IMPL,
+      ProtobufWALStreamReader.class, WALStreamReader.class);
+    Preconditions.checkArgument(
+      AbstractFSWALProvider.Initializer.class.isAssignableFrom(walStreamReaderClass),
+      "The wal stream reader class %s is not a sub class of %s", walStreamReaderClass.getName(),
+      AbstractFSWALProvider.Initializer.class.getName());
     this.conf = conf;
     this.factoryId = factoryId;
     this.excludeDatanodeManager = new ExcludeDatanodeManager(conf);
@@ -301,25 +326,26 @@ public class WALFactory {
     }
   }
 
-  public Reader createReader(final FileSystem fs, final Path path) throws IOException {
-    return createReader(fs, path, (CancelableProgressable) null);
+  public WALStreamReader createStreamReader(FileSystem fs, Path path) throws IOException {
+    return createStreamReader(fs, path, (CancelableProgressable) null);
   }
 
   /**
-   * Create a reader for the WAL. If you are reading from a file that's being written to and need to
-   * reopen it multiple times, use {@link WAL.Reader#reset()} instead of this method then just seek
-   * back to the last known good position.
+   * Create a one-way stream reader for the WAL.
    * @return A WAL reader. Close when done with it.
    */
-  public Reader createReader(final FileSystem fs, final Path path, CancelableProgressable reporter)
-    throws IOException {
-    return createReader(fs, path, reporter, true);
+  public WALStreamReader createStreamReader(FileSystem fs, Path path,
+    CancelableProgressable reporter) throws IOException {
+    return createStreamReader(fs, path, reporter, -1);
   }
 
-  public Reader createReader(final FileSystem fs, final Path path, CancelableProgressable reporter,
-    boolean allowCustom) throws IOException {
-    Class<? extends AbstractFSWALProvider.Reader> lrClass =
-      allowCustom ? logReaderClass : ProtobufLogReader.class;
+  /**
+   * Create a one-way stream reader for the WAL, and start reading from the given
+   * {@code startPosition}.
+   * @return A WAL reader. Close when done with it.
+   */
+  public WALStreamReader createStreamReader(FileSystem fs, Path path,
+    CancelableProgressable reporter, long startPosition) throws IOException {
     try {
       // A wal file could be under recovery, so it may take several
       // tries to get it open. Instead of claiming it is corrupted, retry
@@ -327,22 +353,17 @@ public class WALFactory {
       long startWaiting = EnvironmentEdgeManager.currentTime();
       long openTimeout = timeoutMillis + startWaiting;
       int nbAttempt = 0;
-      AbstractFSWALProvider.Reader reader = null;
+      WALStreamReader reader = null;
       while (true) {
         try {
-          reader = lrClass.getDeclaredConstructor().newInstance();
-          reader.init(fs, path, conf, null);
+          reader = walStreamReaderClass.getDeclaredConstructor().newInstance();
+          ((AbstractFSWALProvider.Initializer) reader).init(fs, path, conf, startPosition);
           return reader;
         } catch (Exception e) {
           // catch Exception so that we close reader for all exceptions. If we don't
           // close the reader, we leak a socket.
           if (reader != null) {
-            try {
-              reader.close();
-            } catch (IOException exception) {
-              LOG.warn("Could not close FSDataInputStream" + exception.getMessage());
-              LOG.debug("exception details", exception);
-            }
+            reader.close();
           }
 
           // Only inspect the Exception to consider retry when it's an IOException
@@ -435,34 +456,30 @@ public class WALFactory {
   }
 
   /**
-   * Create a reader for the given path, accept custom reader classes from conf. If you already have
-   * a WALFactory, you should favor the instance method.
-   * @return a WAL Reader, caller must close.
+   * Create a tailing reader for the given path. Mainly used in replication.
    */
-  public static Reader createReader(final FileSystem fs, final Path path,
-    final Configuration configuration) throws IOException {
-    return getInstance(configuration).createReader(fs, path);
+  public static WALTailingReader createTailingReader(FileSystem fs, Path path, Configuration conf,
+    long startPosition) throws IOException {
+    ProtobufWALTailingReader reader = new ProtobufWALTailingReader();
+    reader.init(fs, path, conf, startPosition);
+    return reader;
   }
 
   /**
-   * Create a reader for the given path, accept custom reader classes from conf. If you already have
-   * a WALFactory, you should favor the instance method.
-   * @return a WAL Reader, caller must close.
+   * Create a one-way stream reader for a given path.
    */
-  static Reader createReader(final FileSystem fs, final Path path,
-    final Configuration configuration, final CancelableProgressable reporter) throws IOException {
-    return getInstance(configuration).createReader(fs, path, reporter);
+  public static WALStreamReader createStreamReader(FileSystem fs, Path path, Configuration conf)
+    throws IOException {
+    return createStreamReader(fs, path, conf, -1);
   }
 
   /**
-   * Create a reader for the given path, ignore custom reader classes from conf. If you already have
-   * a WALFactory, you should favor the instance method. only public pending move of
-   * {@link org.apache.hadoop.hbase.regionserver.wal.Compressor}
-   * @return a WAL Reader, caller must close.
+   * Create a one-way stream reader for a given path.
    */
-  public static Reader createReaderIgnoreCustomClass(final FileSystem fs, final Path path,
-    final Configuration configuration) throws IOException {
-    return getInstance(configuration).createReader(fs, path, null, false);
+  public static WALStreamReader createStreamReader(FileSystem fs, Path path, Configuration conf,
+    long startPosition) throws IOException {
+    return getInstance(conf).createStreamReader(fs, path, (CancelableProgressable) null,
+      startPosition);
   }
 
   /**
