@@ -38,6 +38,7 @@ import org.apache.hadoop.hbase.ipc.HBaseRpcController;
 import org.apache.hadoop.hbase.testclassification.MiscTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.zookeeper.ZKWatcher;
 import org.junit.After;
 import org.junit.Before;
@@ -147,8 +148,7 @@ public class TestMetaTableAccessorNoCluster {
     ZKWatcher zkw =
       new ZKWatcher(UTIL.getConfiguration(), this.getClass().getSimpleName(), ABORTABLE, true);
     // This is a servername we use in a few places below.
-    ServerName sn = ServerName.valueOf("example.com", 1234, System.currentTimeMillis());
-
+    ServerName sn = ServerName.valueOf("example.com", 1234, EnvironmentEdgeManager.currentTime());
     ClusterConnection connection = null;
     try {
       // Mock an ClientProtocol. Our mock implementation will fail a few
