@@ -25,6 +25,7 @@ import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.regex.Pattern;
 import org.apache.hadoop.hbase.CompareOperator;
 import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
@@ -270,6 +271,19 @@ public class TestParseFilter {
     assertEquals("pre*", new String(regexStringComparator.getValue(), StandardCharsets.UTF_8));
   }
 
+  @Test
+  public void testQualifierFilterNoCase() throws IOException {
+    String filterString = "QualifierFilter(=, 'regexstringnocase:pre*')";
+    QualifierFilter qualifierFilter = doTestFilter(filterString, QualifierFilter.class);
+    assertEquals(CompareOperator.EQUAL, qualifierFilter.getCompareOperator());
+    assertTrue(qualifierFilter.getComparator() instanceof RegexStringComparator);
+    RegexStringComparator regexStringComparator =
+      (RegexStringComparator) qualifierFilter.getComparator();
+    assertEquals("pre*", new String(regexStringComparator.getValue(), StandardCharsets.UTF_8));
+    int regexComparatorFlags = regexStringComparator.getEngine().getFlags();
+    assertEquals(Pattern.CASE_INSENSITIVE | Pattern.DOTALL, regexComparatorFlags);
+  }
+
   @Test
   public void testValueFilter() throws IOException {
     String filterString = "ValueFilter(!=, 'substring:pre')";
