@@ -285,14 +285,6 @@ public class TestPrefetch {
     conf.setBoolean(CACHE_DATA_BLOCKS_COMPRESSED_KEY, false);
   }
 
-  @Test
-  public void testPrefetchSkipsRefs() throws Exception {
-    testPrefetchWhenRefs(true, c -> {
-      boolean isCached = c != null;
-      assertFalse(isCached);
-    });
-  }
-
   @Test
   public void testPrefetchDoesntSkipRefs() throws Exception {
     testPrefetchWhenRefs(false, c -> {
