@@ -7,7 +7,7 @@
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,19 +21,18 @@ import static org.apache.hadoop.hbase.AuthUtil.toGroupEntry;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
-import java.util.List;
+import com.google.protobuf.BlockingRpcChannel;
 import java.util.Arrays;
+import java.util.List;
 import java.util.Map;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import java.util.Objects;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HColumnDescriptor;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HTableDescriptor;
 import org.apache.hadoop.hbase.NamespaceDescriptor;
-import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.Admin;
 import org.apache.hadoop.hbase.client.Connection;
@@ -42,24 +41,34 @@ import org.apache.hadoop.hbase.client.Get;
 import org.apache.hadoop.hbase.client.Result;
 import org.apache.hadoop.hbase.client.Table;
 import org.apache.hadoop.hbase.coprocessor.MasterCoprocessorEnvironment;
-import org.apache.hadoop.hbase.coprocessor.ObserverContext;
-import org.apache.hadoop.hbase.protobuf.ProtobufUtil;
+import org.apache.hadoop.hbase.coprocessor.ObserverContextImpl;
+import org.apache.hadoop.hbase.ipc.NettyRpcClientConfigHelper;
 import org.apache.hadoop.hbase.protobuf.generated.AccessControlProtos.AccessControlService;
 import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.security.access.Permission.Action;
+import org.apache.hadoop.hbase.testclassification.MediumTests;
+import org.apache.hadoop.hbase.testclassification.SecurityTests;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.JVMClusterUtil;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.google.common.collect.ListMultimap;
-import com.google.protobuf.BlockingRpcChannel;
+import org.apache.hbase.thirdparty.com.google.common.collect.ListMultimap;
 
-@Category(MediumTests.class)
+@Category({ SecurityTests.class, MediumTests.class })
 public class TestNamespaceCommands extends SecureTestUtil {
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+    HBaseClassTestRule.forClass(TestNamespaceCommands.class);
+
   private static HBaseTestingUtility UTIL = new HBaseTestingUtility();
-  private static final Log LOG = LogFactory.getLog(TestNamespaceCommands.class);
+  private static final Logger LOG = LoggerFactory.getLogger(TestNamespaceCommands.class);
   private static String TEST_NAMESPACE = "ns1";
   private static String TEST_NAMESPACE2 = "ns2";
   private static Configuration conf;
@@ -92,8 +101,8 @@ public class TestNamespaceCommands extends SecureTestUtil {
   private static User USER_NS_EXEC;
 
   // user with rw permissions
-  private static User USER_TABLE_WRITE;  // TODO: WE DO NOT GIVE ANY PERMS TO THIS USER
-  //user with create table permissions alone
+  private static User USER_TABLE_WRITE; // TODO: WE DO NOT GIVE ANY PERMS TO THIS USER
+  // user with create table permissions alone
   private static User USER_TABLE_CREATE; // TODO: WE DO NOT GIVE ANY PERMS TO THIS USER
 
   private static final String GROUP_ADMIN = "group_admin";
@@ -114,7 +123,6 @@ public class TestNamespaceCommands extends SecureTestUtil {
   @BeforeClass
   public static void beforeClass() throws Exception {
     conf = UTIL.getConfiguration();
-    conf.setInt(HConstants.REGION_SERVER_HIGH_PRIORITY_HANDLER_COUNT, 10);
     enableSecurity(conf);
 
     SUPERUSER = User.createUserForTesting(conf, "admin", new String[] { "supergroup" });
@@ -135,41 +143,52 @@ public class TestNamespaceCommands extends SecureTestUtil {
     USER_TABLE_WRITE = User.createUserForTesting(conf, "table_write", new String[0]);
 
     USER_GROUP_ADMIN =
-        User.createUserForTesting(conf, "user_group_admin", new String[] { GROUP_ADMIN });
+      User.createUserForTesting(conf, "user_group_admin", new String[] { GROUP_ADMIN });
     USER_GROUP_NS_ADMIN =
-        User.createUserForTesting(conf, "user_group_ns_admin", new String[] { GROUP_NS_ADMIN });
+      User.createUserForTesting(conf, "user_group_ns_admin", new String[] { GROUP_NS_ADMIN });
     USER_GROUP_CREATE =
-        User.createUserForTesting(conf, "user_group_create", new String[] { GROUP_CREATE });
+      User.createUserForTesting(conf, "user_group_create", new String[] { GROUP_CREATE });
     USER_GROUP_READ =
-        User.createUserForTesting(conf, "user_group_read", new String[] { GROUP_READ });
+      User.createUserForTesting(conf, "user_group_read", new String[] { GROUP_READ });
     USER_GROUP_WRITE =
-        User.createUserForTesting(conf, "user_group_write", new String[] { GROUP_WRITE });
+      User.createUserForTesting(conf, "user_group_write", new String[] { GROUP_WRITE });
     // TODO: other table perms
 
+    UTIL.getConfiguration().setInt(HConstants.HBASE_CLIENT_RETRIES_NUMBER, 2);
     UTIL.startMiniCluster();
     // Wait for the ACL table to become available
-    UTIL.waitTableAvailable(AccessControlLists.ACL_TABLE_NAME.getName(), 30 * 1000);
+    UTIL.waitTableAvailable(PermissionStorage.ACL_TABLE_NAME.getName(), 30 * 1000);
 
-    ACCESS_CONTROLLER = (AccessController) UTIL.getMiniHBaseCluster().getMaster()
-      .getMasterCoprocessorHost()
-        .findCoprocessor(AccessController.class.getName());
+    // Find the Access Controller CP. Could be on master or if master is not serving regions, is
+    // on an arbitrary server.
+    for (JVMClusterUtil.RegionServerThread rst : UTIL.getMiniHBaseCluster()
+      .getLiveRegionServerThreads()) {
+      ACCESS_CONTROLLER = rst.getRegionServer().getRegionServerCoprocessorHost()
+        .findCoprocessor(AccessController.class);
+      if (ACCESS_CONTROLLER != null) {
+        break;
+      }
+    }
+    if (ACCESS_CONTROLLER == null) {
+      throw new NullPointerException();
+    }
 
-    UTIL.getHBaseAdmin().createNamespace(NamespaceDescriptor.create(TEST_NAMESPACE).build());
-    UTIL.getHBaseAdmin().createNamespace(NamespaceDescriptor.create(TEST_NAMESPACE2).build());
+    UTIL.getAdmin().createNamespace(NamespaceDescriptor.create(TEST_NAMESPACE).build());
+    UTIL.getAdmin().createNamespace(NamespaceDescriptor.create(TEST_NAMESPACE2).build());
 
     // grants on global
-    grantGlobal(UTIL, USER_GLOBAL_ADMIN.getShortName(),  Permission.Action.ADMIN);
+    grantGlobal(UTIL, USER_GLOBAL_ADMIN.getShortName(), Permission.Action.ADMIN);
     grantGlobal(UTIL, USER_GLOBAL_CREATE.getShortName(), Permission.Action.CREATE);
-    grantGlobal(UTIL, USER_GLOBAL_WRITE.getShortName(),  Permission.Action.WRITE);
-    grantGlobal(UTIL, USER_GLOBAL_READ.getShortName(),   Permission.Action.READ);
-    grantGlobal(UTIL, USER_GLOBAL_EXEC.getShortName(),   Permission.Action.EXEC);
+    grantGlobal(UTIL, USER_GLOBAL_WRITE.getShortName(), Permission.Action.WRITE);
+    grantGlobal(UTIL, USER_GLOBAL_READ.getShortName(), Permission.Action.READ);
+    grantGlobal(UTIL, USER_GLOBAL_EXEC.getShortName(), Permission.Action.EXEC);
 
     // grants on namespace
-    grantOnNamespace(UTIL, USER_NS_ADMIN.getShortName(),  TEST_NAMESPACE, Permission.Action.ADMIN);
+    grantOnNamespace(UTIL, USER_NS_ADMIN.getShortName(), TEST_NAMESPACE, Permission.Action.ADMIN);
     grantOnNamespace(UTIL, USER_NS_CREATE.getShortName(), TEST_NAMESPACE, Permission.Action.CREATE);
-    grantOnNamespace(UTIL, USER_NS_WRITE.getShortName(),  TEST_NAMESPACE, Permission.Action.WRITE);
-    grantOnNamespace(UTIL, USER_NS_READ.getShortName(),   TEST_NAMESPACE, Permission.Action.READ);
-    grantOnNamespace(UTIL, USER_NS_EXEC.getShortName(),   TEST_NAMESPACE, Permission.Action.EXEC);
+    grantOnNamespace(UTIL, USER_NS_WRITE.getShortName(), TEST_NAMESPACE, Permission.Action.WRITE);
+    grantOnNamespace(UTIL, USER_NS_READ.getShortName(), TEST_NAMESPACE, Permission.Action.READ);
+    grantOnNamespace(UTIL, USER_NS_EXEC.getShortName(), TEST_NAMESPACE, Permission.Action.EXEC);
     grantOnNamespace(UTIL, toGroupEntry(GROUP_NS_ADMIN), TEST_NAMESPACE, Permission.Action.ADMIN);
 
     grantOnNamespace(UTIL, USER_NS_ADMIN.getShortName(), TEST_NAMESPACE2, Permission.Action.ADMIN);
@@ -182,49 +201,45 @@ public class TestNamespaceCommands extends SecureTestUtil {
 
   @AfterClass
   public static void afterClass() throws Exception {
-    UTIL.getHBaseAdmin().deleteNamespace(TEST_NAMESPACE);
-    UTIL.getHBaseAdmin().deleteNamespace(TEST_NAMESPACE2);
+    UTIL.getAdmin().deleteNamespace(TEST_NAMESPACE);
+    UTIL.getAdmin().deleteNamespace(TEST_NAMESPACE2);
     UTIL.shutdownMiniCluster();
   }
 
   @Test
   public void testAclTableEntries() throws Exception {
     String userTestNamespace = "userTestNsp";
-    try(Connection conn = ConnectionFactory.createConnection(conf);
-        Table acl = conn.getTable(AccessControlLists.ACL_TABLE_NAME)) {
-      ListMultimap<String, TablePermission> perms =
-          AccessControlLists.getNamespacePermissions(conf, TEST_NAMESPACE);
-
-      perms = AccessControlLists.getNamespacePermissions(conf, TEST_NAMESPACE);
-      for (Map.Entry<String, TablePermission> entry : perms.entries()) {
-        LOG.debug(entry);
+    Table acl = UTIL.getConnection().getTable(PermissionStorage.ACL_TABLE_NAME);
+    try {
+      ListMultimap<String, UserPermission> perms =
+        PermissionStorage.getNamespacePermissions(conf, TEST_NAMESPACE);
+      for (Map.Entry<String, UserPermission> entry : perms.entries()) {
+        LOG.debug(Objects.toString(entry));
       }
       assertEquals(6, perms.size());
 
       // Grant and check state in ACL table
-      grantOnNamespace(UTIL, userTestNamespace, TEST_NAMESPACE,
-        Permission.Action.WRITE);
+      grantOnNamespace(UTIL, userTestNamespace, TEST_NAMESPACE, Permission.Action.WRITE);
 
       Result result = acl.get(new Get(Bytes.toBytes(userTestNamespace)));
       assertTrue(result != null);
-      perms = AccessControlLists.getNamespacePermissions(conf, TEST_NAMESPACE);
+      perms = PermissionStorage.getNamespacePermissions(conf, TEST_NAMESPACE);
       assertEquals(7, perms.size());
-      List<TablePermission> namespacePerms = perms.get(userTestNamespace);
+      List<UserPermission> namespacePerms = perms.get(userTestNamespace);
       assertTrue(perms.containsKey(userTestNamespace));
       assertEquals(1, namespacePerms.size());
       assertEquals(TEST_NAMESPACE,
-        namespacePerms.get(0).getNamespace());
-      assertEquals(null, namespacePerms.get(0).getFamily());
-      assertEquals(null, namespacePerms.get(0).getQualifier());
-      assertEquals(1, namespacePerms.get(0).getActions().length);
-      assertEquals(Permission.Action.WRITE, namespacePerms.get(0).getActions()[0]);
+        ((NamespacePermission) namespacePerms.get(0).getPermission()).getNamespace());
+      assertEquals(1, namespacePerms.get(0).getPermission().getActions().length);
+      assertEquals(Permission.Action.WRITE, namespacePerms.get(0).getPermission().getActions()[0]);
 
       // Revoke and check state in ACL table
-      revokeFromNamespace(UTIL, userTestNamespace, TEST_NAMESPACE,
-        Permission.Action.WRITE);
+      revokeFromNamespace(UTIL, userTestNamespace, TEST_NAMESPACE, Permission.Action.WRITE);
 
-      perms = AccessControlLists.getNamespacePermissions(conf, TEST_NAMESPACE);
+      perms = PermissionStorage.getNamespacePermissions(conf, TEST_NAMESPACE);
       assertEquals(6, perms.size());
+    } finally {
+      acl.close();
     }
   }
 
@@ -233,7 +248,7 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction modifyNamespace = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        ACCESS_CONTROLLER.preModifyNamespace(ObserverContext.createAndPrepare(CP_ENV, null),
+        ACCESS_CONTROLLER.preModifyNamespace(ObserverContextImpl.createAndPrepare(CP_ENV),
           NamespaceDescriptor.create(TEST_NAMESPACE).addConfiguration("abc", "156").build());
         return null;
       }
@@ -251,7 +266,7 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction createNamespace = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        ACCESS_CONTROLLER.preCreateNamespace(ObserverContext.createAndPrepare(CP_ENV, null),
+        ACCESS_CONTROLLER.preCreateNamespace(ObserverContextImpl.createAndPrepare(CP_ENV),
           NamespaceDescriptor.create(TEST_NAMESPACE2).build());
         return null;
       }
@@ -260,7 +275,7 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction deleteNamespace = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        ACCESS_CONTROLLER.preDeleteNamespace(ObserverContext.createAndPrepare(CP_ENV, null),
+        ACCESS_CONTROLLER.preDeleteNamespace(ObserverContextImpl.createAndPrepare(CP_ENV),
           TEST_NAMESPACE2);
         return null;
       }
@@ -285,7 +300,7 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction getNamespaceAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        ACCESS_CONTROLLER.preGetNamespaceDescriptor(ObserverContext.createAndPrepare(CP_ENV, null),
+        ACCESS_CONTROLLER.preGetNamespaceDescriptor(ObserverContextImpl.createAndPrepare(CP_ENV),
           TEST_NAMESPACE);
         return null;
       }
@@ -303,71 +318,69 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction listAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        try (Connection conn = ConnectionFactory.createConnection(UTIL.getConfiguration());
-            Admin admin = conn.getAdmin()) {
+        Connection unmanagedConnection =
+          ConnectionFactory.createConnection(UTIL.getConfiguration());
+        Admin admin = unmanagedConnection.getAdmin();
+        try {
           return Arrays.asList(admin.listNamespaceDescriptors());
+        } finally {
+          admin.close();
+          unmanagedConnection.close();
         }
       }
     };
 
-    // listNamespaces         : All access*
+    // listNamespaces : All access*
     // * Returned list will only show what you can call getNamespaceDescriptor()
 
     verifyAllowed(listAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_NS_ADMIN, USER_GROUP_ADMIN);
 
     // we have 3 namespaces: [default, hbase, TEST_NAMESPACE, TEST_NAMESPACE2]
-    assertEquals(4, ((List)SUPERUSER.runAs(listAction)).size());
-    assertEquals(4, ((List)USER_GLOBAL_ADMIN.runAs(listAction)).size());
-    assertEquals(4, ((List)USER_GROUP_ADMIN.runAs(listAction)).size());
+    assertEquals(4, ((List) SUPERUSER.runAs(listAction)).size());
+    assertEquals(4, ((List) USER_GLOBAL_ADMIN.runAs(listAction)).size());
+    assertEquals(4, ((List) USER_GROUP_ADMIN.runAs(listAction)).size());
 
-    assertEquals(2, ((List)USER_NS_ADMIN.runAs(listAction)).size());
+    assertEquals(2, ((List) USER_NS_ADMIN.runAs(listAction)).size());
 
-    assertEquals(0, ((List)USER_GLOBAL_CREATE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GLOBAL_WRITE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GLOBAL_READ.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GLOBAL_EXEC.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_NS_CREATE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_NS_WRITE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_NS_READ.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_NS_EXEC.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_TABLE_CREATE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_TABLE_WRITE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GROUP_CREATE.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GROUP_READ.runAs(listAction)).size());
-    assertEquals(0, ((List)USER_GROUP_WRITE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GLOBAL_CREATE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GLOBAL_WRITE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GLOBAL_READ.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GLOBAL_EXEC.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_NS_CREATE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_NS_WRITE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_NS_READ.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_NS_EXEC.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_TABLE_CREATE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_TABLE_WRITE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GROUP_CREATE.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GROUP_READ.runAs(listAction)).size());
+    assertEquals(0, ((List) USER_GROUP_WRITE.runAs(listAction)).size());
   }
 
+  @SuppressWarnings("checkstyle:MethodLength")
   @Test
-  public void testGrantRevoke() throws Exception{
+  public void testGrantRevoke() throws Exception {
     final String testUser = "testUser";
-
+    // Set this else in test context, with limit on the number of threads for
+    // netty eventloopgroup, we can run out of threads if one group used throughout.
+    NettyRpcClientConfigHelper.createEventLoopPerClient(conf);
     // Test if client API actions are authorized
-
     AccessTestAction grantAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        try(Connection conn = ConnectionFactory.createConnection(conf);
-            Table acl = conn.getTable(AccessControlLists.ACL_TABLE_NAME)) {
-          BlockingRpcChannel service =
-              acl.coprocessorService(HConstants.EMPTY_START_ROW);
-          AccessControlService.BlockingInterface protocol =
-            AccessControlService.newBlockingStub(service);
-          ProtobufUtil.grant(null, protocol, testUser, TEST_NAMESPACE, false, Action.WRITE);
+        try (Connection connection = ConnectionFactory.createConnection(conf)) {
+          connection.getAdmin().grant(new UserPermission(testUser,
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.WRITE).build()), false);
         }
         return null;
       }
     };
-
     AccessTestAction grantNamespaceAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        try(Connection conn = ConnectionFactory.createConnection(conf);
-            Table acl = conn.getTable(AccessControlLists.ACL_TABLE_NAME)) {
-          BlockingRpcChannel service = acl.coprocessorService(HConstants.EMPTY_START_ROW);
-          AccessControlService.BlockingInterface protocol =
-            AccessControlService.newBlockingStub(service);
-          ProtobufUtil.grant(null, protocol, USER_GROUP_NS_ADMIN.getShortName(),
-            TEST_NAMESPACE, false, Action.READ);
+        try (Connection conn = ConnectionFactory.createConnection(conf)) {
+          conn.getAdmin().grant(new UserPermission(USER_GROUP_NS_ADMIN.getShortName(),
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.READ).build()), false);
         }
         return null;
       }
@@ -376,33 +389,19 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction revokeAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        try(Connection conn = ConnectionFactory.createConnection(conf);
-            Table acl = conn.getTable(AccessControlLists.ACL_TABLE_NAME)) {
-          BlockingRpcChannel service =
-              acl.coprocessorService(HConstants.EMPTY_START_ROW);
-          AccessControlService.BlockingInterface protocol =
-            AccessControlService.newBlockingStub(service);
-          ProtobufUtil.revoke(null, protocol, testUser, TEST_NAMESPACE, Action.WRITE);
+        try (Connection connection = ConnectionFactory.createConnection(conf)) {
+          connection.getAdmin().revoke(new UserPermission(testUser,
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.WRITE).build()));
         }
         return null;
       }
     };
-
     AccessTestAction revokeNamespaceAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        Connection connection = ConnectionFactory.createConnection(conf);
-        Table acl = connection.getTable(AccessControlLists.ACL_TABLE_NAME);
-        try {
-          BlockingRpcChannel service =
-              acl.coprocessorService(HConstants.EMPTY_START_ROW);
-          AccessControlService.BlockingInterface protocol =
-            AccessControlService.newBlockingStub(service);
-          ProtobufUtil.revoke(null, protocol, USER_GROUP_NS_ADMIN.getShortName(),
-            TEST_NAMESPACE, Action.READ);
-        } finally {
-          acl.close();
-          connection.close();
+        try (Connection connection = ConnectionFactory.createConnection(conf)) {
+          connection.getAdmin().revoke(new UserPermission(USER_GROUP_NS_ADMIN.getShortName(),
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.READ).build()));
         }
         return null;
       }
@@ -411,12 +410,56 @@ public class TestNamespaceCommands extends SecureTestUtil {
     AccessTestAction getPermissionsAction = new AccessTestAction() {
       @Override
       public Object run() throws Exception {
-        try(Connection conn = ConnectionFactory.createConnection(conf);
-            Table acl = conn.getTable(AccessControlLists.ACL_TABLE_NAME)) {
+        try (Connection connection = ConnectionFactory.createConnection(conf)) {
+          connection.getAdmin()
+            .getUserPermissions(GetUserPermissionsRequest.newBuilder(TEST_NAMESPACE).build());
+        }
+        return null;
+      }
+    };
+
+    AccessTestAction preGrantAction = new AccessTestAction() {
+      @Override
+      public Object run() throws Exception {
+        ACCESS_CONTROLLER.preGrant(ObserverContextImpl.createAndPrepare(CP_ENV),
+          new UserPermission(testUser,
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.WRITE).build()),
+          false);
+        return null;
+      }
+    };
+    AccessTestAction preRevokeAction = new AccessTestAction() {
+      @Override
+      public Object run() throws Exception {
+        ACCESS_CONTROLLER.preRevoke(ObserverContextImpl.createAndPrepare(CP_ENV),
+          new UserPermission(testUser,
+            Permission.newBuilder(TEST_NAMESPACE).withActions(Action.WRITE).build()));
+        return null;
+      }
+    };
+
+    AccessTestAction grantCPAction = new AccessTestAction() {
+      @Override
+      public Object run() throws Exception {
+        try (Connection connection = ConnectionFactory.createConnection(conf);
+          Table acl = connection.getTable(PermissionStorage.ACL_TABLE_NAME)) {
           BlockingRpcChannel service = acl.coprocessorService(HConstants.EMPTY_START_ROW);
           AccessControlService.BlockingInterface protocol =
             AccessControlService.newBlockingStub(service);
-          ProtobufUtil.getUserPermissions(null, protocol, Bytes.toBytes(TEST_NAMESPACE));
+          AccessControlUtil.grant(null, protocol, testUser, TEST_NAMESPACE, false, Action.WRITE);
+        }
+        return null;
+      }
+    };
+    AccessTestAction revokeCPAction = new AccessTestAction() {
+      @Override
+      public Object run() throws Exception {
+        try (Connection connection = ConnectionFactory.createConnection(conf);
+          Table acl = connection.getTable(PermissionStorage.ACL_TABLE_NAME)) {
+          BlockingRpcChannel service = acl.coprocessorService(HConstants.EMPTY_START_ROW);
+          AccessControlService.BlockingInterface protocol =
+            AccessControlService.newBlockingStub(service);
+          AccessControlUtil.revoke(null, protocol, testUser, TEST_NAMESPACE, Action.WRITE);
         }
         return null;
       }
@@ -426,7 +469,6 @@ public class TestNamespaceCommands extends SecureTestUtil {
     verifyDenied(grantAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
       USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
       USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
-
     verifyAllowed(grantNamespaceAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN,
       USER_NS_ADMIN, USER_GROUP_NS_ADMIN);
     verifyDenied(grantNamespaceAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
@@ -437,7 +479,6 @@ public class TestNamespaceCommands extends SecureTestUtil {
     verifyDenied(revokeAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
       USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
       USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
-
     verifyAllowed(revokeNamespaceAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN,
       USER_NS_ADMIN, USER_GROUP_NS_ADMIN);
     verifyDenied(revokeNamespaceAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
@@ -449,6 +490,24 @@ public class TestNamespaceCommands extends SecureTestUtil {
     verifyDenied(getPermissionsAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
       USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
       USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
+
+    verifyAllowed(preGrantAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN, USER_NS_ADMIN);
+    verifyDenied(preGrantAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
+      USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
+      USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
+    verifyAllowed(preRevokeAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN, USER_NS_ADMIN);
+    verifyDenied(preRevokeAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
+      USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
+      USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
+
+    verifyAllowed(grantCPAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN, USER_NS_ADMIN);
+    verifyDenied(grantCPAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
+      USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
+      USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
+    verifyAllowed(revokeCPAction, SUPERUSER, USER_GLOBAL_ADMIN, USER_GROUP_ADMIN, USER_NS_ADMIN);
+    verifyDenied(revokeCPAction, USER_GLOBAL_CREATE, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
+      USER_GLOBAL_EXEC, USER_NS_CREATE, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
+      USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_CREATE);
   }
 
   @Test
@@ -458,15 +517,16 @@ public class TestNamespaceCommands extends SecureTestUtil {
       public Object run() throws Exception {
         HTableDescriptor htd = new HTableDescriptor(TableName.valueOf(TEST_TABLE));
         htd.addFamily(new HColumnDescriptor(TEST_FAMILY));
-        ACCESS_CONTROLLER.preCreateTable(ObserverContext.createAndPrepare(CP_ENV, null), htd, null);
+        ACCESS_CONTROLLER.preCreateTable(ObserverContextImpl.createAndPrepare(CP_ENV), htd, null);
         return null;
       }
     };
 
-    //createTable            : superuser | global(C) | NS(C)
-    verifyAllowed(createTable, SUPERUSER, USER_GLOBAL_CREATE, USER_NS_CREATE, USER_GROUP_CREATE);
-    verifyDenied(createTable, USER_GLOBAL_ADMIN, USER_GLOBAL_WRITE, USER_GLOBAL_READ,
-      USER_GLOBAL_EXEC, USER_NS_ADMIN, USER_NS_WRITE, USER_NS_READ, USER_NS_EXEC,
-      USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ, USER_GROUP_WRITE, USER_GROUP_ADMIN);
+    // createTable : superuser | global(AC) | NS(AC)
+    verifyAllowed(createTable, SUPERUSER, USER_GLOBAL_CREATE, USER_NS_CREATE, USER_GROUP_CREATE,
+      USER_GLOBAL_ADMIN, USER_NS_ADMIN, USER_GROUP_ADMIN);
+    verifyDenied(createTable, USER_GLOBAL_WRITE, USER_GLOBAL_READ, USER_GLOBAL_EXEC, USER_NS_WRITE,
+      USER_NS_READ, USER_NS_EXEC, USER_TABLE_CREATE, USER_TABLE_WRITE, USER_GROUP_READ,
+      USER_GROUP_WRITE);
   }
 }
