@@ -293,14 +293,6 @@ public class TestLoadIncrementalHFiles {
     runTest(testName, htd, preCreateTable, tableSplitKeys, hfileRanges, useMap, false, depth);
   }
 
-  public static int loadHFiles(String testName, TableDescriptor htd, HBaseTestingUtility util,
-    byte[] fam, byte[] qual, boolean preCreateTable, byte[][] tableSplitKeys,
-    byte[][][] hfileRanges, boolean useMap, boolean deleteFile, boolean copyFiles, int initRowCount,
-    int factor) throws Exception {
-    return loadHFiles(testName, htd, util, fam, qual, preCreateTable, tableSplitKeys, hfileRanges,
-      useMap, deleteFile, copyFiles, initRowCount, factor, 2);
-  }
-
   public static int loadHFiles(String testName, TableDescriptor htd, HBaseTestingUtility util,
     byte[] fam, byte[] qual, boolean preCreateTable, byte[][] tableSplitKeys,
     byte[][][] hfileRanges, boolean useMap, boolean deleteFile, boolean copyFiles, int initRowCount,
