@@ -108,7 +108,7 @@ public class HFileTestUtil {
       .withDataBlockEncoding(encoding).withColumnFamily(family).build();
     HFile.Writer writer = HFile.getWriterFactory(configuration, new CacheConfig(configuration))
       .withPath(fs, path).withFileContext(meta).create();
-    long now = System.currentTimeMillis();
+    long now = EnvironmentEdgeManager.currentTime();
     try {
       // subtract 2 since iterateOnSplits doesn't include boundary keys
       for (byte[] key : Bytes.iterateOnSplits(startKey, endKey, numRows - 2)) {
@@ -127,7 +127,7 @@ public class HFileTestUtil {
         writer.append(kv);
       }
     } finally {
-      writer.appendFileInfo(BULKLOAD_TIME_KEY, Bytes.toBytes(System.currentTimeMillis()));
+      writer.appendFileInfo(BULKLOAD_TIME_KEY, Bytes.toBytes(EnvironmentEdgeManager.currentTime()));
       writer.close();
     }
   }
