@@ -17,6 +17,10 @@
  */
 package org.apache.hadoop.hbase.wal;
 
+import static org.apache.hadoop.hbase.replication.master.ReplicationSinkTrackerTableCreator.REPLICATION_SINK_TRACKER_TABLE_NAME;
+import static org.apache.hadoop.hbase.replication.regionserver.ReplicationMarkerChore.getRowKey;
+import static org.apache.hadoop.hbase.wal.WALEdit.METAFAMILY;
+import static org.apache.hadoop.hbase.wal.WALEdit.REPLICATION_MARKER;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
@@ -60,12 +64,12 @@ import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.RegionInfoBuilder;
 import org.apache.hadoop.hbase.coordination.SplitLogWorkerCoordination;
+import org.apache.hadoop.hbase.master.SplitLogManager;
 import org.apache.hadoop.hbase.regionserver.HRegion;
-import org.apache.hadoop.hbase.regionserver.LastSequenceId;
-import org.apache.hadoop.hbase.regionserver.RegionServerServices;
-import org.apache.hadoop.hbase.regionserver.wal.FaultyProtobufLogReader;
+import org.apache.hadoop.hbase.regionserver.wal.AbstractProtobufWALReader;
+import org.apache.hadoop.hbase.regionserver.wal.FaultyProtobufWALStreamReader;
 import org.apache.hadoop.hbase.regionserver.wal.InstrumentedLogWriter;
-import org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader;
+import org.apache.hadoop.hbase.replication.regionserver.ReplicationMarkerChore;
 import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.testclassification.LargeTests;
 import org.apache.hadoop.hbase.testclassification.RegionServerTests;
@@ -75,7 +79,6 @@ import org.apache.hadoop.hbase.util.CommonFSUtils;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.Threads;
 import org.apache.hadoop.hbase.wal.WAL.Entry;
-import org.apache.hadoop.hbase.wal.WAL.Reader;
 import org.apache.hadoop.hbase.wal.WALProvider.Writer;
 import org.apache.hadoop.hbase.wal.WALSplitter.CorruptedLogFileException;
 import org.apache.hadoop.hdfs.DFSTestUtil;
@@ -102,7 +105,6 @@ import org.apache.hbase.thirdparty.com.google.common.collect.ImmutableMap;
 import org.apache.hbase.thirdparty.com.google.protobuf.ByteString;
 
 import org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil;
-import org.apache.hadoop.hbase.shaded.protobuf.generated.ClusterStatusProtos;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.WALProtos;
 
 /**
@@ -153,7 +155,8 @@ public class TestWALSplit {
   @BeforeClass
   public static void setUpBeforeClass() throws Exception {
     conf = TEST_UTIL.getConfiguration();
-    conf.setClass("hbase.regionserver.hlog.writer.impl", InstrumentedLogWriter.class, Writer.class);
+    conf.set(WALFactory.WAL_PROVIDER, "filesystem");
+    conf.setClass(FSHLogProvider.WRITER_IMPL, InstrumentedLogWriter.class, Writer.class);
     // This is how you turn off shortcircuit read currently. TODO: Fix. Should read config.
     System.setProperty("hbase.tests.use.shortcircuit.reads", "false");
     // Create fake maping user to group and set it to the conf.
@@ -369,70 +372,6 @@ public class TestWALSplit {
     }
   }
 
-  // If another worker is assigned to split a WAl and last worker is still running, both should not
-  // impact each other's progress
-  @Test
-  public void testTwoWorkerSplittingSameWAL() throws IOException, InterruptedException {
-    int numWriter = 1, entries = 10;
-    generateWALs(numWriter, entries, -1, 0);
-    FileStatus logfile = fs.listStatus(WALDIR)[0];
-    FileSystem spiedFs = Mockito.spy(fs);
-    RegionServerServices zombieRSServices = Mockito.mock(RegionServerServices.class);
-    RegionServerServices newWorkerRSServices = Mockito.mock(RegionServerServices.class);
-    Mockito.when(zombieRSServices.getServerName())
-      .thenReturn(ServerName.valueOf("zombie-rs.abc.com,1234,1234567890"));
-    Mockito.when(newWorkerRSServices.getServerName())
-      .thenReturn(ServerName.valueOf("worker-rs.abc.com,1234,1234569870"));
-    Thread zombieWorker = new SplitWALWorker(logfile, spiedFs, zombieRSServices);
-    Thread newWorker = new SplitWALWorker(logfile, spiedFs, newWorkerRSServices);
-    zombieWorker.start();
-    newWorker.start();
-    newWorker.join();
-    zombieWorker.join();
-
-    for (String region : REGIONS) {
-      Path[] logfiles = getLogForRegion(TABLE_NAME, region);
-      assertEquals("wrong number of split files for region", numWriter, logfiles.length);
-
-      int count = 0;
-      for (Path lf : logfiles) {
-        count += countWAL(lf);
-      }
-      assertEquals("wrong number of edits for region " + region, entries, count);
-    }
-  }
-
-  private class SplitWALWorker extends Thread implements LastSequenceId {
-    final FileStatus logfile;
-    final FileSystem fs;
-    final RegionServerServices rsServices;
-
-    public SplitWALWorker(FileStatus logfile, FileSystem fs, RegionServerServices rsServices) {
-      super(rsServices.getServerName().toShortString());
-      setDaemon(true);
-      this.fs = fs;
-      this.logfile = logfile;
-      this.rsServices = rsServices;
-    }
-
-    @Override
-    public void run() {
-      try {
-        boolean ret =
-          WALSplitter.splitLogFile(HBASEDIR, logfile, fs, conf, null, this, null, wals, rsServices);
-        assertTrue("Both splitting should pass", ret);
-      } catch (IOException e) {
-        LOG.warn(getName() + " Worker exiting " + e);
-      }
-    }
-
-    @Override
-    public ClusterStatusProtos.RegionStoreSequenceIds getLastSequenceId(byte[] encodedRegionName) {
-      return ClusterStatusProtos.RegionStoreSequenceIds.newBuilder()
-        .setLastFlushedSequenceId(HConstants.NO_SEQNUM).build();
-    }
-  }
-
   /**
    * @see "https://issues.apache.org/jira/browse/HBASE-3020"
    */
@@ -464,7 +403,7 @@ public class TestWALSplit {
   private Path createRecoveredEditsPathForRegion() throws IOException {
     byte[] encoded = RegionInfoBuilder.FIRST_META_REGIONINFO.getEncodedNameAsBytes();
     Path p = WALSplitUtil.getRegionSplitEditsPath(TableName.META_TABLE_NAME, encoded, 1,
-      FILENAME_BEING_SPLIT, TMPDIRNAME, conf, "");
+      FILENAME_BEING_SPLIT, TMPDIRNAME, conf);
     return p;
   }
 
@@ -548,6 +487,39 @@ public class TestWALSplit {
     assertEquals(11, countWAL(splitLog[0]));
   }
 
+  /**
+   * Tests that WalSplitter ignores replication marker edits.
+   */
+  @Test
+  public void testSplitRemovesReplicationMarkerEdits() throws IOException {
+    RegionInfo regionInfo = ReplicationMarkerChore.REGION_INFO;
+    Path path = new Path(WALDIR, WAL_FILE_PREFIX + "1");
+    generateReplicationMarkerEdits(path, regionInfo);
+    useDifferentDFSClient();
+    List<FileStatus> logFiles =
+      SplitLogManager.getFileList(conf, Collections.singletonList(WALDIR), null);
+    assertEquals(1, logFiles.size());
+    assertEquals(path, logFiles.get(0).getPath());
+    List<Path> splitPaths = WALSplitter.split(HBASELOGDIR, WALDIR, OLDLOGDIR, fs, conf, wals);
+    // Make sure that WALSplitter doesn't fail.
+    assertEquals(0, splitPaths.size());
+  }
+
+  private void generateReplicationMarkerEdits(Path path, RegionInfo regionInfo) throws IOException {
+    long timestamp = EnvironmentEdgeManager.currentTime();
+    fs.mkdirs(WALDIR);
+    try (Writer writer = wals.createWALWriter(fs, path)) {
+      WALProtos.ReplicationMarkerDescriptor.Builder builder =
+        WALProtos.ReplicationMarkerDescriptor.newBuilder();
+      builder.setWalName("wal-name");
+      builder.setRegionServerName("rs-name");
+      builder.setOffset(0L);
+      WALProtos.ReplicationMarkerDescriptor desc = builder.build();
+      appendEntry(writer, REPLICATION_SINK_TRACKER_TABLE_NAME, regionInfo.getEncodedNameAsBytes(),
+        getRowKey(desc.getRegionServerName(), timestamp), METAFAMILY, REPLICATION_MARKER, VALUE, 1);
+    }
+  }
+
   /**
    * @param expectedEntries -1 to not assert
    * @return the count across all regions
@@ -634,10 +606,11 @@ public class TestWALSplit {
   @Test
   public void testCorruptedFileGetsArchivedIfSkipErrors() throws IOException {
     conf.setBoolean(WALSplitter.SPLIT_SKIP_ERRORS_KEY, true);
-    List<FaultyProtobufLogReader.FailureType> failureTypes =
-      Arrays.asList(FaultyProtobufLogReader.FailureType.values()).stream()
-        .filter(x -> x != FaultyProtobufLogReader.FailureType.NONE).collect(Collectors.toList());
-    for (FaultyProtobufLogReader.FailureType failureType : failureTypes) {
+    List<FaultyProtobufWALStreamReader.FailureType> failureTypes =
+      Arrays.asList(FaultyProtobufWALStreamReader.FailureType.values()).stream()
+        .filter(x -> x != FaultyProtobufWALStreamReader.FailureType.NONE)
+        .collect(Collectors.toList());
+    for (FaultyProtobufWALStreamReader.FailureType failureType : failureTypes) {
       final Set<String> walDirContents = splitCorruptWALs(failureType);
       final Set<String> archivedLogs = new HashSet<>();
       final StringBuilder archived = new StringBuilder("Archived logs in CORRUPTDIR:");
@@ -655,14 +628,14 @@ public class TestWALSplit {
    * @return set of wal names present prior to split attempt.
    * @throws IOException if the split process fails
    */
-  private Set<String> splitCorruptWALs(final FaultyProtobufLogReader.FailureType failureType)
+  private Set<String> splitCorruptWALs(final FaultyProtobufWALStreamReader.FailureType failureType)
     throws IOException {
-    Class<?> backupClass = conf.getClass("hbase.regionserver.hlog.reader.impl", Reader.class);
+    String backupClass = conf.get(WALFactory.WAL_STREAM_READER_CLASS_IMPL);
     InstrumentedLogWriter.activateFailure = false;
 
     try {
-      conf.setClass("hbase.regionserver.hlog.reader.impl", FaultyProtobufLogReader.class,
-        Reader.class);
+      conf.setClass(WALFactory.WAL_STREAM_READER_CLASS_IMPL, FaultyProtobufWALStreamReader.class,
+        WALStreamReader.class);
       conf.set("faultyprotobuflogreader.failuretype", failureType.name());
       // Clean up from previous tests or previous loop
       try {
@@ -691,21 +664,25 @@ public class TestWALSplit {
       WALSplitter.split(HBASELOGDIR, WALDIR, OLDLOGDIR, fs, conf, wals);
       return walDirContents;
     } finally {
-      conf.setClass("hbase.regionserver.hlog.reader.impl", backupClass, Reader.class);
+      if (backupClass != null) {
+        conf.set(WALFactory.WAL_STREAM_READER_CLASS_IMPL, backupClass);
+      } else {
+        conf.unset(WALFactory.WAL_STREAM_READER_CLASS_IMPL);
+      }
     }
   }
 
   @Test(expected = IOException.class)
   public void testTrailingGarbageCorruptionLogFileSkipErrorsFalseThrows() throws IOException {
     conf.setBoolean(WALSplitter.SPLIT_SKIP_ERRORS_KEY, false);
-    splitCorruptWALs(FaultyProtobufLogReader.FailureType.BEGINNING);
+    splitCorruptWALs(FaultyProtobufWALStreamReader.FailureType.BEGINNING);
   }
 
   @Test
   public void testCorruptedLogFilesSkipErrorsFalseDoesNotTouchLogs() throws IOException {
     conf.setBoolean(WALSplitter.SPLIT_SKIP_ERRORS_KEY, false);
     try {
-      splitCorruptWALs(FaultyProtobufLogReader.FailureType.BEGINNING);
+      splitCorruptWALs(FaultyProtobufWALStreamReader.FailureType.BEGINNING);
     } catch (IOException e) {
       LOG.debug("split with 'skip errors' set to 'false' correctly threw");
     }
@@ -732,13 +709,12 @@ public class TestWALSplit {
     assertEquals(1, splitLog.length);
 
     int actualCount = 0;
-    Reader in = wals.createReader(fs, splitLog[0]);
-    @SuppressWarnings("unused")
-    Entry entry;
-    while ((entry = in.next()) != null)
-      ++actualCount;
+    try (WALStreamReader in = wals.createStreamReader(fs, splitLog[0])) {
+      while (in.next() != null) {
+        ++actualCount;
+      }
+    }
     assertEquals(expectedCount, actualCount);
-    in.close();
 
     // should not have stored the EOF files as corrupt
     FileStatus[] archivedLogs =
@@ -1075,15 +1051,17 @@ public class TestWALSplit {
 
       /* Produce a mock reader that generates fake entries */
       @Override
-      protected Reader getReader(FileStatus file, boolean skipErrors,
+      protected WALStreamReader getReader(FileStatus file, boolean skipErrors,
         CancelableProgressable reporter) throws IOException, CorruptedLogFileException {
-        Reader mockReader = Mockito.mock(Reader.class);
+        WALStreamReader mockReader = Mockito.mock(WALStreamReader.class);
         Mockito.doAnswer(new Answer<Entry>() {
           int index = 0;
 
           @Override
           public Entry answer(InvocationOnMock invocation) throws Throwable {
-            if (index >= numFakeEdits) return null;
+            if (index >= numFakeEdits) {
+              return null;
+            }
 
             // Generate r0 through r4 in round robin fashion
             int regionIdx = index % regions.size();
@@ -1382,8 +1360,8 @@ public class TestWALSplit {
       case TRUNCATE:
         fs.delete(path, false);
         out = fs.create(path);
-        out.write(corrupted_bytes, 0,
-          fileSize - (32 + ProtobufLogReader.PB_WAL_COMPLETE_MAGIC.length + Bytes.SIZEOF_INT));
+        out.write(corrupted_bytes, 0, fileSize
+          - (32 + AbstractProtobufWALReader.PB_WAL_COMPLETE_MAGIC.length + Bytes.SIZEOF_INT));
         closeOrFlush(close, out);
         break;
 
@@ -1422,11 +1400,11 @@ public class TestWALSplit {
 
   private int countWAL(Path log) throws IOException {
     int count = 0;
-    Reader in = wals.createReader(fs, log);
-    while (in.next() != null) {
-      count++;
+    try (WALStreamReader in = wals.createStreamReader(fs, log)) {
+      while (in.next() != null) {
+        count++;
+      }
     }
-    in.close();
     return count;
   }
 
@@ -1503,7 +1481,8 @@ public class TestWALSplit {
   }
 
   private boolean logsAreEqual(Path p1, Path p2) throws IOException {
-    try (Reader in1 = wals.createReader(fs, p1); Reader in2 = wals.createReader(fs, p2)) {
+    try (WALStreamReader in1 = wals.createStreamReader(fs, p1);
+      WALStreamReader in2 = wals.createStreamReader(fs, p2)) {
       Entry entry1;
       Entry entry2;
       while ((entry1 = in1.next()) != null) {
