@@ -155,6 +155,11 @@ public class ThriftAdmin implements Admin {
     return listTableDescriptors((Pattern) null);
   }
 
+  @Override
+  public List<TableDescriptor> listTableDescriptorsByState(boolean isEnabled) throws IOException {
+    throw new NotImplementedException("listTableDescriptorsByState not supported in ThriftAdmin");
+  }
+
   @Override
   public HTableDescriptor[] listTables(Pattern pattern) throws IOException {
     String regex = (pattern == null ? null : pattern.toString());
@@ -236,6 +241,11 @@ public class ThriftAdmin implements Admin {
     }
   }
 
+  @Override
+  public List<TableName> listTableNamesByState(boolean isEnabled) throws IOException {
+    throw new NotImplementedException("listTableNamesByState not supported in ThriftAdmin");
+  }
+
   @Override
   public HTableDescriptor getTableDescriptor(TableName tableName)
     throws TableNotFoundException, IOException {
@@ -878,6 +888,24 @@ public class ThriftAdmin implements Admin {
     throw new NotImplementedException("split not supported in ThriftAdmin");
   }
 
+  @Override
+  public void truncateRegion(byte[] regionName) throws IOException {
+    throw new NotImplementedException("Truncate Region not supported in ThriftAdmin");
+  }
+
+  @Override
+  public Future<Void> truncateRegionAsync(byte[] regionName) {
+    throw new NotImplementedException("Truncate Region Async not supported in ThriftAdmin");
+  }
+
+  /**
+   * Get the list of cached files
+   */
+  @Override
+  public List<String> getCachedFilesList(ServerName serverName) {
+    throw new NotImplementedException("getCachedFilesList not supported in ThriftAdmin");
+  }
+
   @Override
   public void splitRegion(byte[] regionName, byte[] splitPoint) {
     throw new NotImplementedException("splitRegion not supported in ThriftAdmin");
@@ -1280,6 +1308,11 @@ public class ThriftAdmin implements Admin {
 
   }
 
+  @Override
+  public boolean isReplicationPeerEnabled(String peerId) throws IOException {
+    throw new NotImplementedException("isReplicationPeerEnabled not supported in ThriftAdmin");
+  }
+
   @Override
   public void clearCompactionQueues(ServerName serverName, Set<String> queues) {
     throw new NotImplementedException("clearCompactionQueues not supported in ThriftAdmin");
@@ -1328,6 +1361,11 @@ public class ThriftAdmin implements Admin {
     throw new NotImplementedException("disableTableAsync not supported in ThriftAdmin");
   }
 
+  @Override
+  public Future<Void> flushAsync(TableName tableName, List<byte[]> columnFamilies) {
+    throw new NotImplementedException("flushAsync not supported in ThriftAdmin");
+  }
+
   @Override
   public Pair<Integer, Integer> getAlterStatus(TableName tableName) {
     throw new NotImplementedException("getAlterStatus not supported in ThriftAdmin");
@@ -1466,4 +1504,22 @@ public class ThriftAdmin implements Admin {
     throw new NotImplementedException(
       "modifyTableStoreFileTrackerAsync not supported in ThriftAdmin");
   }
+
+  @Override
+  public boolean replicationPeerModificationSwitch(boolean on, boolean drainProcedures)
+    throws IOException {
+    throw new NotImplementedException(
+      "replicationPeerModificationSwitch not supported in ThriftAdmin");
+  }
+
+  @Override
+  public void flush(TableName tableName, List<byte[]> columnFamilies) {
+    throw new NotImplementedException("flush not supported in ThriftAdmin");
+  }
+
+  @Override
+  public boolean isReplicationPeerModificationEnabled() throws IOException {
+    throw new NotImplementedException(
+      "isReplicationPeerModificationEnabled not supported in ThriftAdmin");
+  }
 }
